@@ -1,4 +1,4 @@
-#include "FileZilla.h"
+#include <filezilla.h>
 #include "tlssocket.h"
 #include "ControlSocket.h"
 
@@ -6,6 +6,20 @@
 #include <gnutls/x509.h>
 #include <errno.h>
 
+//#define TLSDEBUG 1
+#if TLSDEBUG
+// This is quite ugly
+CControlSocket* pLoggingControlSocket;
+void log_func(int level, const char* msg)
+{
+	if (!msg)
+		return;
+	wxString s(msg, wxConvLocal);
+	s.Trim();
+	pLoggingControlSocket->LogMessage(Debug_Debug, _T("tls: %d %s"), level, s.c_str());
+}
+#endif
+
 CTlsSocket::CTlsSocket(CSocketEventHandler* pEvtHandler, CSocket* pSocket, CControlSocket* pOwner)
 	: CBackend(pEvtHandler), m_pOwner(pOwner)
 {
@@ -40,6 +54,8 @@ CTlsSocket::CTlsSocket(CSocketEventHandler* pEvtHandler, CSocket* pSocket, CCont
 	m_shutdown_requested = false;
 
 	m_socket_eof = false;
+
+	m_require_root_trust = false;
 }
 
 CTlsSocket::~CTlsSocket()
@@ -60,6 +76,11 @@ bool CTlsSocket::Init()
 		return false;
 	}
 
+#if TLSDEBUG
+	pLoggingControlSocket = m_pOwner;
+	gnutls_global_set_log_function(log_func);
+	gnutls_global_set_log_level(99);
+#endif
 	res = gnutls_certificate_allocate_credentials(&m_certCredentials);
 	if (res < 0)
 	{
@@ -139,6 +160,8 @@ void CTlsSocket::Uninit()
 
 	delete [] m_implicitTrustedCert.data;
 	m_implicitTrustedCert.data = 0;
+
+	m_require_root_trust = false;
 }
 
 void CTlsSocket::LogError(int code)
@@ -155,10 +178,10 @@ void CTlsSocket::LogError(int code)
 #else
 		wxString str(error);
 #endif
-		m_pOwner->LogMessage(::Debug_Warning, _T("GnuTLS error %d: %s"), code, str.c_str());
+		m_pOwner->LogMessage(::Error, _T("GnuTLS error %d: %s"), code, str.c_str());
 	}
 	else
-		m_pOwner->LogMessage(::Debug_Warning, _T("GnuTLS error %d"), code);
+		m_pOwner->LogMessage(::Error, _T("GnuTLS error %d"), code);
 }
 
 void CTlsSocket::PrintAlert()
@@ -190,6 +213,9 @@ ssize_t CTlsSocket::PullFunction(gnutls_transport_ptr_t ptr, void* data, size_t
 
 ssize_t CTlsSocket::PushFunction(const void* data, size_t len)
 {
+#if TLSDEBUG
+	m_pOwner->LogMessage(Debug_Debug, _T("CTlsSocket::PushFunction(%x, %d)"), data, len);
+#endif
 	if (!m_canWriteToSocket)
 	{
 		gnutls_transport_set_errno(m_session, EAGAIN);
@@ -218,11 +244,18 @@ ssize_t CTlsSocket::PushFunction(const void* data, size_t len)
 		return -1;
 	}
 
+#if TLSDEBUG
+	m_pOwner->LogMessage(Debug_Debug, _T("  returning %d"), written);
+#endif
+
 	return written;
 }
 
 ssize_t CTlsSocket::PullFunction(void* data, size_t len)
 {
+#if TLSDEBUG
+	m_pOwner->LogMessage(Debug_Debug, _T("CTlsSocket::PullFunction(%x, %d)"), data, len);
+#endif
 	if (!m_pSocketBackend)
 	{
 		gnutls_transport_set_errno(m_session, 0);
@@ -265,6 +298,10 @@ ssize_t CTlsSocket::PullFunction(void* data, size_t len)
 	if (!read)
 		m_socket_eof = true;
 
+#if TLSDEBUG
+	m_pOwner->LogMessage(Debug_Debug, _T("  returning %d"), read);
+#endif
+
 	return read;
 }
 
@@ -329,7 +366,7 @@ void CTlsSocket::OnRead()
 		return;
 
 	if (m_tlsState == handshake)
-		Handshake();
+		ContinueHandshake();
 	if (m_tlsState == closing)
 		ContinueShutdown();
 	else if (m_tlsState == conn)
@@ -353,7 +390,7 @@ void CTlsSocket::OnSend()
 		return;
 
 	if (m_tlsState == handshake)
-		Handshake();
+		ContinueHandshake();
 	else if (m_tlsState == closing)
 		ContinueShutdown();
 	else if (m_tlsState == conn)
@@ -425,6 +462,15 @@ int CTlsSocket::Handshake(const CTlsSocket* pPrimarySocket /*=0*/, bool try_resu
 			CopySessionData(pPrimarySocket);
 	}
 
+	return ContinueHandshake();
+}
+
+int CTlsSocket::ContinueHandshake()
+{
+	m_pOwner->LogMessage(Debug_Verbose, _T("CTlsSocket::ContinueHandshake()"));
+	wxASSERT(m_session);
+	wxASSERT(m_tlsState == handshake);
+
 	int res = gnutls_handshake(m_session);
 	if (!res)
 	{
@@ -457,7 +503,7 @@ int CTlsSocket::Handshake(const CTlsSocket* pPrimarySocket /*=0*/, bool try_resu
 	else if (res == GNUTLS_E_AGAIN || res == GNUTLS_E_INTERRUPTED)
 		return FZ_REPLY_WOULDBLOCK;
 
-	Failure(res, 0);
+	Failure(res, ECONNABORTED);
 
 	return FZ_REPLY_ERROR;
 }
@@ -673,6 +719,7 @@ void CTlsSocket::CheckResumeFailedReadWrite()
 
 void CTlsSocket::Failure(int code, int socket_error)
 {
+	m_pOwner->LogMessage(::Debug_Debug, _T("CTlsSocket::Failure(%d, %d)"), code, socket_error);
 	if (code)
 	{
 		LogError(code);
@@ -814,7 +861,7 @@ int CTlsSocket::VerifyCertificate()
 {
 	if (m_tlsState != handshake)
 	{
-		m_pOwner->LogMessage(Debug_Warning, _T("VerifyCertificate called at wrong time"));
+		m_pOwner->LogMessage(::Debug_Warning, _T("VerifyCertificate called at wrong time"));
 		return FZ_REPLY_ERROR;
 	}
 
@@ -822,7 +869,36 @@ int CTlsSocket::VerifyCertificate()
 
 	if (gnutls_certificate_type_get(m_session) != GNUTLS_CRT_X509)
 	{
-		m_pOwner->LogMessage(::Debug_Warning, _T("Unsupported certificate type"));
+		m_pOwner->LogMessage(::Error, _("Unsupported certificate type"));
+		Failure(0, ECONNABORTED);
+		return FZ_REPLY_ERROR;
+	}
+
+	unsigned int status = 0;
+	if (gnutls_certificate_verify_peers2(m_session, &status) < 0)
+	{
+		m_pOwner->LogMessage(::Error, _("Failed to verify peer certificate"));
+		Failure(0, ECONNABORTED);
+		return FZ_REPLY_ERROR;
+	}
+
+	if (status & GNUTLS_CERT_REVOKED)
+	{
+		m_pOwner->LogMessage(::Error, _("Beware! Certificate has been revoked"));
+		Failure(0, ECONNABORTED);
+		return FZ_REPLY_ERROR;
+	}
+
+	if (status & GNUTLS_CERT_SIGNER_NOT_CA)
+	{
+		m_pOwner->LogMessage(::Error, _("Incomplete chain, top certificate is not self-signed certificate authority certificate"));
+		Failure(0, ECONNABORTED);
+		return FZ_REPLY_ERROR;
+	}
+
+	if (m_require_root_trust && status & GNUTLS_CERT_SIGNER_NOT_FOUND)
+	{
+		m_pOwner->LogMessage(::Error, _("Root certificate is not trusted"));
 		Failure(0, ECONNABORTED);
 		return FZ_REPLY_ERROR;
 	}
@@ -831,7 +907,7 @@ int CTlsSocket::VerifyCertificate()
 	const gnutls_datum_t* cert_list = gnutls_certificate_get_peers(m_session, &cert_list_size);
 	if (!cert_list || !cert_list_size)
 	{
-		m_pOwner->LogMessage(::Debug_Warning, _T("gnutls_certificate_get_peers returned no certificates"));
+		m_pOwner->LogMessage(::Error, _("gnutls_certificate_get_peers returned no certificates"));
 		Failure(0, ECONNABORTED);
 		return FZ_REPLY_ERROR;
 	}
@@ -859,14 +935,14 @@ int CTlsSocket::VerifyCertificate()
 		gnutls_x509_crt_t cert;
 		if (gnutls_x509_crt_init(&cert))
 		{
-			m_pOwner->LogMessage(::Debug_Warning, _T("gnutls_x509_crt_init failed"));
+			m_pOwner->LogMessage(::Error, _("Could not initialize structure for peer certificates, gnutls_x509_crt_init failed"));
 			Failure(0, ECONNABORTED);
 			return FZ_REPLY_ERROR;
 		}
 
 		if (gnutls_x509_crt_import(cert, cert_list, GNUTLS_X509_FMT_DER))
 		{
-			m_pOwner->LogMessage(::Debug_Warning, _T("gnutls_x509_crt_import failed"));
+			m_pOwner->LogMessage(::Error, _("Could not import peer certificates, gnutls_x509_crt_import failed"));
 			Failure(0, ECONNABORTED);
 			gnutls_x509_crt_deinit(cert);
 			return FZ_REPLY_ERROR;
@@ -913,7 +989,7 @@ int CTlsSocket::VerifyCertificate()
 			LogError(res);
 		if (subject == _T(""))
 		{
-			m_pOwner->LogMessage(::Debug_Warning, _T("gnutls_x509_get_dn failed"));
+			m_pOwner->LogMessage(::Error, _("Could not get distinguished name of certificate subject, gnutls_x509_get_dn failed"));
 			Failure(0, ECONNABORTED);
 			gnutls_x509_crt_deinit(cert);
 			return FZ_REPLY_ERROR;
@@ -938,7 +1014,7 @@ int CTlsSocket::VerifyCertificate()
 			LogError(res);
 		if (issuer == _T(""))
 		{
-			m_pOwner->LogMessage(::Debug_Warning, _T("gnutls_x509_get_issuer_dn failed"));
+			m_pOwner->LogMessage(::Error, _("Could not get distinguished name of certificate issuer, gnutls_x509_get_issuer_dn failed"));
 			Failure(0, ECONNABORTED);
 			gnutls_x509_crt_deinit(cert);
 			return FZ_REPLY_ERROR;
@@ -1009,3 +1085,22 @@ wxString CTlsSocket::GetMacName()
 	else
 		return _T("unknown");
 }
+
+bool CTlsSocket::AddTrustedRootCertificate(const wxString& cert)
+{
+	if (!m_initialized)
+		return false;
+
+	if (cert == _T(""))
+		return false;
+
+	m_require_root_trust = true;
+
+	const wxWX2MBbuf str = cert.mb_str();
+
+	gnutls_datum_t datum;
+	datum.data = (unsigned char*)(const char*)str;
+	datum.size = strlen(str);
+
+	return gnutls_certificate_set_x509_trust_mem(m_certCredentials, &datum, GNUTLS_X509_FMT_PEM) > 0;
+}
