@@ -1,4 +1,7 @@
-#include "FileZilla.h"
+#include <filezilla.h>
+#ifdef _MSC_VER
+#pragma hdrstop
+#endif
 #include "filezillaapp.h"
 #include "Mainfrm.h"
 #include "Options.h"
@@ -9,6 +12,8 @@
 #endif
 #include "xmlfunctions.h"
 #include <wx/tokenzr.h>
+#include "cmdline.h"
+#include "welcome_dialog.h"
 
 #include <wx/xrc/xh_bmpbt.h>
 #include <wx/xrc/xh_bttn.h>
@@ -31,44 +36,53 @@
 #include <wx/xrc/xh_stlin.h>
 #include <wx/xrc/xh_sttxt.h>
 #include <wx/xrc/xh_text.h>
-#include <wx/xrc/xh_toolb.h>
 #include <wx/xrc/xh_tree.h>
 #include <wx/xrc/xh_hyperlink.h>
+#include "xh_toolb_ex.h"
+#include "xh_menu_ex.h"
+#ifdef __WXMSW__
+#include <wx/socket.h>
+#include <wx/dynlib.h>
+#endif
+#ifdef WITH_LIBDBUS
+#include <../dbus/session_manager.h>
+#endif
 
 #if defined(__WXMAC__) || defined(__UNIX__)
 #include <wx/stdpaths.h>
 #endif
 
+#ifdef __WXGTK__
+#include "locale_initializer.h"
+#endif
+
 #ifdef ENABLE_BINRELOC
 	#define BR_PTHREADS 0
 	#include "prefix.h"
 #endif
 
-#ifdef __WXMSW__
-	#include <shlobj.h>
-
-	// Needed for MinGW:
-	#ifndef SHGFP_TYPE_CURRENT
-		#define SHGFP_TYPE_CURRENT 0
-	#endif
-#endif
-
 #ifdef _DEBUG
 #define new DEBUG_NEW
 #endif
 
+#ifndef __WXGTK__
 IMPLEMENT_APP(CFileZillaApp);
+#else
+IMPLEMENT_APP_NO_MAIN(CFileZillaApp);
+#endif //__WXGTK__
 
 CFileZillaApp::CFileZillaApp()
 {
 	m_pWrapEngine = 0;
 	m_pLocale = 0;
+	m_pCommandLine = 0;
 }
 
 CFileZillaApp::~CFileZillaApp()
 {
 	delete m_pLocale;
 	delete m_pWrapEngine;
+	delete m_pCommandLine;
 	COptions::Destroy();
 }
 
@@ -133,29 +147,129 @@ bool CheckForWin2003FirewallBug()
 
 	return true;
 }
+
+extern "C" 
+{
+	typedef HRESULT (WINAPI *t_SetCurrentProcessExplicitAppUserModelID)(PCWSTR AppID);
+}
+
+static void SetAppId()
+{
+	wxDynamicLibrary dll;
+	if (!dll.Load(_T("shell32.dll")))
+		return;
+
+	if (!dll.HasSymbol(_T("SetCurrentProcessExplicitAppUserModelID")))
+		return;
+
+	t_SetCurrentProcessExplicitAppUserModelID pSetCurrentProcessExplicitAppUserModelID =
+		(t_SetCurrentProcessExplicitAppUserModelID)dll.GetSymbol(_T("SetCurrentProcessExplicitAppUserModelID"));
+
+	if (!pSetCurrentProcessExplicitAppUserModelID)
+		return;
+
+	pSetCurrentProcessExplicitAppUserModelID(_T("FileZilla.Client.AppID"));
+}
+
 #endif //__WXMSW__
 
 bool CFileZillaApp::OnInit()
 {
+	srand( (unsigned)time( NULL ) );
+
 #if wxUSE_DEBUGREPORT && wxUSE_ON_FATAL_EXCEPTION
 	//wxHandleFatalExceptions();
 #endif
-	wxApp::OnInit();
 
+#ifdef __WXMSW__
+	// Need to call WSAStartup. Let wx do that for us
+	wxSocketBase::Initialize();
+
+	SetAppId();
+#endif
+
+	//wxSystemOptions is slow, if a value is not set, it keeps querying the environment
+	//each and every time...
+	wxSystemOptions::SetOption(_T("filesys.no-mimetypesmanager"), 0);
+	wxSystemOptions::SetOption(_T("window-default-variant"), _T(""));
+#ifdef __WXMSW__
+	wxSystemOptions::SetOption(_T("no-maskblt"), 0);
+	wxSystemOptions::SetOption(_T("msw.window.no-clip-children"), 0);
+	wxSystemOptions::SetOption(_T("msw.font.no-proof-quality"), 0);
+#endif
+
+#ifdef __WXMSW__
 	wxSystemOptions::SetOption(_T("msw.remap"), 0);
+#endif
+#ifdef __WXMAC__
 	wxSystemOptions::SetOption(_T("mac.listctrl.always_use_generic"), 1);
+#endif
+
+	int cmdline_result = ProcessCommandLine();
+	if (!cmdline_result)
+		return false;
 
 	LoadLocales();
+
+	if (cmdline_result < 0)
+	{
+		m_pCommandLine->DisplayUsage();
+		return false;
+	}
+
 	InitDefaultsDir();
-    InitSettingsDir();
 
 	COptions::Init();
+
 	wxString language = COptions::Get()->GetOption(OPTION_LANGUAGE);
+	const wxLanguageInfo* pInfo = wxLocale::FindLanguageInfo(language);
 	if (language != _T(""))
 	{
-		const wxLanguageInfo* pInfo = wxLocale::FindLanguageInfo(language);
+#ifdef __WXGTK__
+		if (CInitializer::error)
+		{
+			wxString error;
+
+			wxLocale *loc = wxGetLocale();
+			const wxLanguageInfo* currentInfo = loc ? loc->GetLanguageInfo(loc->GetLanguage()) : 0;
+			if (!loc || !currentInfo)
+			{
+				if (!pInfo)
+					error.Printf(_("Failed to set language to %s, using default system language."),
+						language.c_str());
+				else
+					error.Printf(_("Failed to set language to %s (%s), using default system language."),
+						pInfo->Description.c_str(), language.c_str());
+			}
+			else
+			{
+				wxString currentName = currentInfo->CanonicalName;
+
+				if (!pInfo)
+					error.Printf(_("Failed to set language to %s, using default system language (%s, %s)."),
+						language.c_str(), loc->GetLocale(),
+						currentName.c_str());
+				else
+					error.Printf(_("Failed to set language to %s (%s), using default system language (%s, %s)."),
+						pInfo->Description.c_str(), language.c_str(), loc->GetLocale(),
+						currentName.c_str());
+			}
+
+			error += _T("\n");
+			error += _("Please make sure the requested locale is installed on your system.");
+			wxMessageBox(error, _("Failed to change language"), wxICON_EXCLAMATION);
+
+			COptions::Get()->SetOption(OPTION_LANGUAGE, _T(""));
+		}
+#else
 		if (!pInfo || !SetLocale(pInfo->Language))
-			wxMessageBox(wxString::Format(_("Failed to set language to %s, using default system language"), language.c_str()), _("Failed to change language"), wxICON_EXCLAMATION);
+		{
+			if (pInfo && pInfo->Description)
+				wxMessageBox(wxString::Format(_("Failed to set language to %s (%s), using default system language"), pInfo->Description.c_str(), language.c_str()), _("Failed to change language"), wxICON_EXCLAMATION);
+			else
+				wxMessageBox(wxString::Format(_("Failed to set language to %s, using default system language"), language.c_str()), _("Failed to change language"), wxICON_EXCLAMATION);
+		}
+#endif
 	}
 
 #ifndef _DEBUG
@@ -168,19 +282,6 @@ DO NOT use it in production environments,\r\n\
 DO NOT distribute the binaries,\r\n\
 DO NOT complain about it\r\n\
 USE AT OWN RISK"), _T("Important Information"));
-	else if (buildType == _T("official"))
-	{
-		wxString greetingVersion = COptions::Get()->GetOption(OPTION_GREETINGVERSION);
-		if (greetingVersion == _T("") ||
-			CBuildInfo::ConvertToVersionNumber(CBuildInfo::GetVersion()) > CBuildInfo::ConvertToVersionNumber(greetingVersion))
-		{
-			COptions::Get()->SetOption(OPTION_GREETINGVERSION, CBuildInfo::GetVersion());
-
-			wxMessageBox(wxString::Format(_T("Welcome to FileZilla %s\r\n\r\n\
-Please report all bugs you may encounter.\
-"), CBuildInfo::GetVersion().c_str()), _T("Welcome"));
-		}
-	}
 #endif
 
 	if (!LoadResourceFiles())
@@ -212,17 +313,40 @@ FileZilla will timeout on big transfers.\
 
 	wxUpdateUIEvent::SetMode(wxUPDATE_UI_PROCESS_SPECIFIED);
 
+#ifdef WITH_LIBDBUS
+	CSessionManager::Init();
+#endif
+
 	// Load the text wrapping engine
 	m_pWrapEngine = new CWrapEngine();
 	m_pWrapEngine->LoadCache();
 
-	wxFrame *frame = new CMainFrame();
+	CMainFrame *frame = new CMainFrame();
 	frame->Show(true);
 	SetTopWindow(frame);
 
+	CWelcomeDialog *welcome_dialog = new CWelcomeDialog;
+	welcome_dialog->Run(frame, false, true);
+
+	frame->ProcessCommandLine();
+	frame->PostInitialize();
+
 	return true;
 }
 
+int CFileZillaApp::OnExit()
+{
+	COptions::Get()->SaveIfNeeded();
+
+#ifdef WITH_LIBDBUS
+	CSessionManager::Uninit();
+#endif
+#ifdef __WXMSW__
+	wxSocketBase::Shutdown();
+#endif
+	return wxApp::OnExit();
+}
+
 bool CFileZillaApp::FileExists(const wxString& file) const
 {
 	int pos = file.Find('*');
@@ -260,8 +384,18 @@ wxString CFileZillaApp::GetDataDir(wxString fileToFind) const
 	 * Basically we just check a couple of paths for presence of the resources,
 	 * and hope we find them. If not, the user can still specify on the cmdline
 	 * and using environment variables where the resources are.
+	 *
+	 * At least on OS X it's simple: All inside application bundle.
 	 */
 
+#ifdef __WXMAC__
+	wxString path = wxStandardPaths::Get().GetDataDir();
+	if (FileExists(path + fileToFind))
+		return path;
+
+	return _T("");
+#else
+
 	wxPathList pathList;
 	// FIXME: --datadir cmdline
 
@@ -270,7 +404,6 @@ wxString CFileZillaApp::GetDataDir(wxString fileToFind) const
 
 	// Next try the current path and the current executable path.
 	// Without this, running development versions would be difficult.
-
 	pathList.Add(wxGetCwd());
 
 #ifdef ENABLE_BINRELOC
@@ -299,13 +432,12 @@ wxString CFileZillaApp::GetDataDir(wxString fileToFind) const
 		wxFileName fn(path);
 		pathList.Add(fn.GetPath());
 	}
-#endif
+#endif //ENABLE_BINRELOC and __WXMSW__ blocks
 
 	// Now scan through the path
 	pathList.AddEnvList(_T("PATH"));
 
-#ifdef __WXMSW__
-#else
+#ifndef __WXMSW__
 	// Try some common paths
 	pathList.Add(_T("/usr/share/filezilla"));
 	pathList.Add(_T("/usr/local/share/filezilla"));
@@ -322,10 +454,6 @@ wxString CFileZillaApp::GetDataDir(wxString fileToFind) const
 			return cur + _T("/share/filezilla");
 		if (FileExists(cur + _T("/filezilla") + fileToFind))
 			return cur + _T("/filezilla");
-#if defined(__WXMAC__) || defined(__WXCOCOA__)
-		if (FileExists(cur + _T("/FileZilla.app/Contents/SharedSupport") + fileToFind))
-			return cur + _T("/FileZilla.app/Contents/SharedSupport");
-#endif
 	}
 
 	for (node = pathList.begin(); node != pathList.end(); node++)
@@ -345,6 +473,7 @@ wxString CFileZillaApp::GetDataDir(wxString fileToFind) const
 	}
 
 	return _T("");
+#endif //__WXMAC__
 }
 
 bool CFileZillaApp::LoadResourceFiles()
@@ -371,31 +500,31 @@ bool CFileZillaApp::LoadResourceFiles()
 	pResource->SetFlags(pResource->GetFlags() | wxXRC_NO_RELOADING);
 #endif
 
-    pResource->AddHandler(new wxMenuXmlHandler);
-    pResource->AddHandler(new wxMenuBarXmlHandler);
+	pResource->AddHandler(new wxMenuXmlHandler);
+	pResource->AddHandler(new wxMenuBarXmlHandlerEx);
 	pResource->AddHandler(new wxDialogXmlHandler);
 	pResource->AddHandler(new wxPanelXmlHandler);
 	pResource->AddHandler(new wxSizerXmlHandler);
 	pResource->AddHandler(new wxButtonXmlHandler);
 	pResource->AddHandler(new wxBitmapButtonXmlHandler);
-    pResource->AddHandler(new wxStaticTextXmlHandler);
-    pResource->AddHandler(new wxStaticBoxXmlHandler);
-    pResource->AddHandler(new wxStaticBitmapXmlHandler);
-    pResource->AddHandler(new wxTreeCtrlXmlHandler);
-    pResource->AddHandler(new wxListCtrlXmlHandler);
-    pResource->AddHandler(new wxCheckListBoxXmlHandler);
-    pResource->AddHandler(new wxChoiceXmlHandler);
-    pResource->AddHandler(new wxGaugeXmlHandler);
-    pResource->AddHandler(new wxCheckBoxXmlHandler);
-    pResource->AddHandler(new wxSpinCtrlXmlHandler);
-    pResource->AddHandler(new wxRadioButtonXmlHandler);
-    pResource->AddHandler(new wxNotebookXmlHandler);
-    pResource->AddHandler(new wxTextCtrlXmlHandler);
-    pResource->AddHandler(new wxListBoxXmlHandler);
-    pResource->AddHandler(new wxToolBarXmlHandler);
+	pResource->AddHandler(new wxStaticTextXmlHandler);
+	pResource->AddHandler(new wxStaticBoxXmlHandler);
+	pResource->AddHandler(new wxStaticBitmapXmlHandler);
+	pResource->AddHandler(new wxTreeCtrlXmlHandler);
+	pResource->AddHandler(new wxListCtrlXmlHandler);
+	pResource->AddHandler(new wxCheckListBoxXmlHandler);
+	pResource->AddHandler(new wxChoiceXmlHandler);
+	pResource->AddHandler(new wxGaugeXmlHandler);
+	pResource->AddHandler(new wxCheckBoxXmlHandler);
+	pResource->AddHandler(new wxSpinCtrlXmlHandler);
+	pResource->AddHandler(new wxRadioButtonXmlHandler);
+	pResource->AddHandler(new wxNotebookXmlHandler);
+	pResource->AddHandler(new wxTextCtrlXmlHandler);
+	pResource->AddHandler(new wxListBoxXmlHandler);
+	pResource->AddHandler(new wxToolBarXmlHandlerEx);
 	pResource->AddHandler(new wxStaticLineXmlHandler);
-    pResource->AddHandler(new wxScrolledWindowXmlHandler);
-    pResource->AddHandler(new wxHyperlinkCtrlXmlHandler);
+	pResource->AddHandler(new wxScrolledWindowXmlHandler);
+	pResource->AddHandler(new wxHyperlinkCtrlXmlHandler);
 
 	wxString resourceDir = m_resourceDir;
 #if wxUSE_FILESYSTEM
@@ -419,106 +548,17 @@ bool CFileZillaApp::InitDefaultsDir()
 		m_defaultsDir = _T("/etc/filezilla");
 	else
 #endif
-		m_defaultsDir = GetDataDir(_T("/fzdefaults.xml"));
+	m_defaultsDir = GetDataDir(_T("/fzdefaults.xml"));
 
 	return m_defaultsDir != _T("");
 }
 
-wxString CFileZillaApp::GetSettingsDirFromDefaults()
-{
-	if (GetDefaultsDir() == _T(""))
-		return _T("");
-
-	wxFileName fn(GetDefaultsDir(), _T("fzdefaults.xml"));
-	if (!fn.IsOk() || !fn.FileExists())
-		return _T("");
-
-	CXmlFile file(fn);
-	TiXmlElement* element = file.Load();
-	if (!element)
-		return _T("");
-
-	TiXmlElement* settings = element->FirstChildElement("Settings");
-	if (!settings)
-		return _T("");
-
-	TiXmlElement* setting = FindElementWithAttribute(settings, "Setting", "name", "Config Location");
-	if (!setting)
-		return _T("");
-
-	wxString location = GetTextElement(setting);
-	if (location == _T(""))
-		return _T("");
-
-	wxStringTokenizer tokenizer(location, _T("/\\"), wxTOKEN_RET_EMPTY_ALL);
-	location = _T("");
-	while (tokenizer.HasMoreTokens())
-	{
-		wxString token = tokenizer.GetNextToken();
-		if (token[0] == '$')
-		{
-			if (token[1] == '$')
-				token = token.Mid(1);
-			else
-			{
-				wxString value;
-				if (wxGetEnv(token.Mid(1), &value))
-					token = value;
-			}
-		}
-		location += token;
-		const wxChar delimiter = tokenizer.GetLastDelimiter();
-		if (delimiter)
-			location += delimiter;
-
-	}
-
-	wxFileName norm(location, _T(""));
-	norm.Normalize(wxPATH_NORM_ALL, fn.GetPath());
-
-	location = norm.GetFullPath();
-	return location;
-}
-
-bool CFileZillaApp::InitSettingsDir()
-{
-	m_settingsDir = GetSettingsDirFromDefaults();
-
-	wxFileName fn;
-	if (m_settingsDir != _T(""))
-		fn = wxFileName(m_settingsDir, _T(""));
-	else
-	{
-
-#ifdef __WXMSW__
-		wxChar buffer[MAX_PATH * 2 + 1];
-
-		if (SUCCEEDED(SHGetFolderPath(0, CSIDL_APPDATA, 0, SHGFP_TYPE_CURRENT, buffer)))
-		{
-			fn = wxFileName(buffer, _T(""));
-			fn.AppendDir(_T("FileZilla"));
-		}
-		else
-		{
-			// Fall back to directory where the executable is
-			if (GetModuleFileName(0, buffer, MAX_PATH * 2))
-				fn = buffer;
-		}
-#else
-		fn = wxFileName(wxGetHomeDir(), _T(""));
-		fn.AppendDir(_T(".filezilla"));
-#endif
-	}
-	if (!fn.DirExists())
-		wxMkdir(fn.GetPath(), 0700);
-	m_settingsDir = fn.GetPath();
-
-	return true;
-}
-
 bool CFileZillaApp::LoadLocales()
 {
+#ifndef __WXMAC__
 	m_localesDir = GetDataDir(_T("/../locale/*/filezilla.mo"));
+	if (m_localesDir == _T(""))
+		m_localesDir = GetDataDir(_T("/../locale/*/LC_MESSAGES/filezilla.mo"));
 	if (m_localesDir != _T(""))
 	{
 		if (m_localesDir[m_localesDir.Length() - 1] != wxFileName::GetPathSeparator())
@@ -537,6 +577,9 @@ bool CFileZillaApp::LoadLocales()
 			m_localesDir += _T("locales");
 		}
 	}
+#else
+	m_localesDir = wxStandardPaths::Get().GetDataDir() + _T("/locales");
+#endif
 
 	if (m_localesDir != _T(""))
 	{
@@ -591,6 +634,13 @@ int CFileZillaApp::GetCurrentLanguage() const
 	return m_pLocale->GetLanguage();
 }
 
+wxString CFileZillaApp::GetCurrentLanguageCode() const
+{
+	if (!m_pLocale)
+		return _T("");
+
+	return m_pLocale->GetCanonicalName();
+}
 
 #if wxUSE_DEBUGREPORT && wxUSE_ON_FATAL_EXCEPTION
 
@@ -641,6 +691,21 @@ void CFileZillaApp::CheckExistsFzsftp()
 {
 	// Get the correct path to the fzsftp executable
 
+#ifdef __WXMAC__
+	wxString executable = wxStandardPaths::Get().GetExecutablePath();
+	int pos = executable.Find('/', true);
+	if (pos != -1)
+		executable = executable.Left(pos);
+	executable += _T("/fzsftp");
+	if (!wxFileName::FileExists(executable))
+	{
+		wxMessageBox(wxString::Format(_("%s could not be found. Without this component of FileZilla, SFTP will not work.\n\nPlease download FileZilla again. If this problem persists, please submit a bug report."), executable.c_str()),
+			_("File not found"), wxICON_ERROR);
+		executable.clear();
+	}
+
+#else
+
 	wxString program = _T("fzsftp");
 #ifdef __WXMSW__
 	program += _T(".exe");
@@ -693,16 +758,7 @@ void CFileZillaApp::CheckExistsFzsftp()
 
 	if (!found)
 	{
-#ifdef __WXMAC__
-		// Get application path within the bundle
-		wxFileName fn(wxStandardPaths::Get().GetDataDir() + _T("/../MacOS/"), program);
-		fn.Normalize();
-		if (fn.FileExists())
-		{
-			executable = fn.GetFullPath();
-			found = true;
-		}
-#elif defined(__UNIX__)
+#ifdef __UNIX__
 		const wxString prefix = ((const wxStandardPaths&)wxStandardPaths::Get()).GetInstallPrefix();
 		if (prefix != _T("/usr/local"))
 		{
@@ -739,7 +795,56 @@ void CFileZillaApp::CheckExistsFzsftp()
 	{
 		wxMessageBox(wxString::Format(_("%s could not be found. Without this component of FileZilla, SFTP will not work.\n\nPossible solutions:\n- Make sure %s is in a directory listed in your PATH environment variable.\n- Set the full path to %s in the FZ_FZSFTP environment variable."), program.c_str(), program.c_str(), program.c_str()),
 			_("File not found"), wxICON_ERROR);
+		executable.clear();
 	}
-	else
-		COptions::Get()->SetOption(OPTION_FZSFTP_EXECUTABLE, executable);
+#endif
+
+	// Quote path if it contains spaces
+	if (executable.Find(_T(" ")) != -1 && executable[0] != '"' && executable[0] != '\'')
+		executable = _T("\"") + executable + _T("\"");
+
+	COptions::Get()->SetOption(OPTION_FZSFTP_EXECUTABLE, executable);
+}
+
+#ifdef __WXMSW__
+extern "C" BOOL CALLBACK EnumWindowCallback(HWND hwnd, LPARAM lParam)
+{
+	HWND child = FindWindowEx(hwnd, 0, 0, _T("FileZilla process identificator 3919DB0A-082D-4560-8E2F-381A35969FB4"));
+	if (child)
+	{
+		::PostMessage(hwnd, WM_ENDSESSION, (WPARAM)TRUE, (LPARAM)ENDSESSION_LOGOFF);
+	}
+
+	return TRUE;
+}
+#endif
+
+int CFileZillaApp::ProcessCommandLine()
+{
+	m_pCommandLine = new CCommandLine(argc, argv);
+	int res = m_pCommandLine->Parse() ? 1 : -1;
+
+	if (res > 0)
+	{
+		if (m_pCommandLine->HasSwitch(CCommandLine::close))
+		{
+#ifdef __WXMSW__
+			EnumWindows((WNDENUMPROC)EnumWindowCallback, 0);
+#endif
+			return 0;
+		}
+
+		if (m_pCommandLine->HasSwitch(CCommandLine::version))
+		{
+			wxString out = wxString::Format(_T("FileZilla %s"), CBuildInfo::GetVersion().c_str());
+			if (!CBuildInfo::GetBuildType().empty())
+				out += _T(" ") + CBuildInfo::GetBuildType() + _T(" build");
+			out += _T(", compiled on ") + CBuildInfo::GetBuildDateString();
+
+			printf("%s\n", (const char*)out.mb_str());
+			return 0;
+		}
+	}
+
+	return res;
 }
