@@ -1,6 +1,7 @@
+#include <filezilla.h>
+
 #define FILELISTCTRL_INCLUDE_TEMPLATE_DEFINITION
 
-#include "FileZilla.h"
 #include "RemoteListView.h"
 #include "commandqueue.h"
 #include "queue.h"
@@ -16,7 +17,7 @@
 #include "edithandler.h"
 #include "dragdropmanager.h"
 #include <wx/clipbrd.h>
-
+#include "sizeformatting.h"
 #ifdef __WXMSW__
 #include "shellapi.h"
 #include "commctrl.h"
@@ -83,7 +84,7 @@ public:
 				{
 					if (index == (int)m_pRemoteListView->m_pDirectoryListing->GetCount())
 						subdir = _T("..");
-					else if ((*m_pRemoteListView->m_pDirectoryListing)[index].dir)
+					else if ((*m_pRemoteListView->m_pDirectoryListing)[index].is_dir())
 						subdir = (*m_pRemoteListView->m_pDirectoryListing)[index].name;
 				}
 			}
@@ -117,7 +118,7 @@ public:
 			{
 				if (index == (int)m_pRemoteListView->m_pDirectoryListing->GetCount())
 					subdir = _T("..");
-				else if ((*m_pRemoteListView->m_pDirectoryListing)[index].dir)
+				else if ((*m_pRemoteListView->m_pDirectoryListing)[index].is_dir())
 					subdir = (*m_pRemoteListView->m_pDirectoryListing)[index].name;
 			}
 		}
@@ -149,7 +150,7 @@ public:
 				dir.AddSegment(info.name);
 				if (dir == target || dir.IsParentOf(target, false))
 				{
-					wxMessageBox(_("A directory cannot be dragged into one if its subdirectories."));
+					wxMessageBox(_("A directory cannot be dragged into one of its subdirectories."));
 					return wxDragNone;
 				}
 			}
@@ -195,7 +196,7 @@ public:
 				hit = -1;
 			else if (index != (int)m_pRemoteListView->m_pDirectoryListing->GetCount())
 			{
-				if (!(*m_pRemoteListView->m_pDirectoryListing)[index].dir)
+				if (!(*m_pRemoteListView->m_pDirectoryListing)[index].is_dir())
 					hit = -1;
 				else
 				{
@@ -277,16 +278,15 @@ class CInfoText : public wxWindow
 public:
 	CInfoText(wxWindow* parent, const wxString& text)
 		: wxWindow(parent, wxID_ANY, wxPoint(0, 60), wxDefaultSize),
-		m_text(_T("<") + text + _T(">"))
+		m_text(text)
 	{
 		SetForegroundColour(parent->GetForegroundColour());
 		SetBackgroundColour(parent->GetBackgroundColour());
 		GetTextExtent(m_text, &m_textSize.x, &m_textSize.y);
 	}
 
-	void SetText(wxString text)
+	void SetText(const wxString &text)
 	{
-		text = _T("<") + text + _T(">");
 		if (text == m_text)
 			return;
 
@@ -330,23 +330,22 @@ BEGIN_EVENT_TABLE(CRemoteListView, CFileListCtrl<CGenericFileData>)
 	EVT_MENU(XRCID("ID_RENAME"), CRemoteListView::OnMenuRename)
 	EVT_MENU(XRCID("ID_CHMOD"), CRemoteListView::OnMenuChmod)
 	EVT_KEY_DOWN(CRemoteListView::OnKeyDown)
-	EVT_LIST_BEGIN_LABEL_EDIT(wxID_ANY, CRemoteListView::OnBeginLabelEdit)
-	EVT_LIST_END_LABEL_EDIT(wxID_ANY, CRemoteListView::OnEndLabelEdit)
 	EVT_SIZE(CRemoteListView::OnSize)
 	EVT_LIST_BEGIN_DRAG(wxID_ANY, CRemoteListView::OnBeginDrag)
 	EVT_MENU(XRCID("ID_EDIT"), CRemoteListView::OnMenuEdit)
 	EVT_MENU(XRCID("ID_ENTER"), CRemoteListView::OnMenuEnter)
 	EVT_MENU(XRCID("ID_GETURL"), CRemoteListView::OnMenuGeturl)
+	EVT_MENU(XRCID("ID_CONTEXT_REFRESH"), CRemoteListView::OnMenuRefresh)
 END_EVENT_TABLE()
 
 CRemoteListView::CRemoteListView(wxWindow* pParent, CState *pState, CQueueView* pQueue)
 	: CFileListCtrl<CGenericFileData>(pParent, pState, pQueue),
-	CSystemImageList(16),
 	CStateEventHandler(pState)
 {
 	pState->RegisterHandler(this, STATECHANGE_REMOTE_DIR);
 	pState->RegisterHandler(this, STATECHANGE_REMOTE_DIR_MODIFIED);
 	pState->RegisterHandler(this, STATECHANGE_APPLYFILTER);
+	pState->RegisterHandler(this, STATECHANGE_REMOTE_LINKNOTDIR);
 
 	m_dropTarget = -1;
 
@@ -355,7 +354,7 @@ CRemoteListView::CRemoteListView(wxWindow* pParent, CState *pState, CQueueView*
 
 	const unsigned long widths[6] = { 80, 75, 80, 100, 80, 80 };
 
-	AddColumn(_("Filename"), wxLIST_FORMAT_LEFT, widths[0]);
+	AddColumn(_("Filename"), wxLIST_FORMAT_LEFT, widths[0], true);
 	AddColumn(_("Filesize"), wxLIST_FORMAT_RIGHT, widths[1]);
 	AddColumn(_("Filetype"), wxLIST_FORMAT_LEFT, widths[2]);
 	AddColumn(_("Last modified"), wxLIST_FORMAT_LEFT, widths[3]);
@@ -368,9 +367,7 @@ CRemoteListView::CRemoteListView(wxWindow* pParent, CState *pState, CQueueView*
 	m_dirIcon = GetIconIndex(dir);
 	SetImageList(GetSystemImageList(), wxIMAGE_LIST_SMALL);
 
-#ifdef __WXMSW__
 	InitHeaderImageList();
-#endif
 
 	SetDirectoryListing(0);
 
@@ -381,6 +378,7 @@ CRemoteListView::CRemoteListView(wxWindow* pParent, CState *pState, CQueueView*
 	EnablePrefixSearch(true);
 
 	m_pLinkResolveState = 0;
+
 }
 
 CRemoteListView::~CRemoteListView()
@@ -404,7 +402,7 @@ int CRemoteListView::OnGetItemImage(long item) const
 	if (icon != -2)
 		return icon;
 
-	icon = pThis->GetIconIndex(file, (*m_pDirectoryListing)[index].name, false);
+	icon = pThis->GetIconIndex(file, (*m_pDirectoryListing)[index].name, false, (*m_pDirectoryListing)[index].is_dir());
 	return icon;
 }
 
@@ -452,15 +450,24 @@ void CRemoteListView::UpdateDirectoryListing_Added(const CSharedPointer<const CD
 		const CDirentry& entry = (*pDirectoryListing)[i];
 		CGenericFileData data;
 		data.flags = normal;
-		data.icon = entry.dir ? m_dirIcon : -2;
+		if (entry.is_dir())
+		{
+			data.icon = m_dirIcon;
+#ifndef __WXMSW__
+			if (entry.is_link())
+				data.icon += 3;
+#endif
+		}
+		else
+			data.icon = -2;
 		m_fileData.push_back(data);
 
-		if (filter.FilenameFiltered(entry.name, path, entry.dir, entry.size, false, 0))
+		if (filter.FilenameFiltered(entry.name, path, entry.is_dir(), entry.size, false, 0, entry.has_date() ? &entry.time : 0))
 			continue;
 
 		if (m_pFilelistStatusBar)
 		{
-			if (entry.dir)
+			if (entry.is_dir())
 				m_pFilelistStatusBar->AddDirectory();
 			else
 				m_pFilelistStatusBar->AddFile(entry.size);
@@ -514,7 +521,7 @@ void CRemoteListView::UpdateDirectoryListing_Added(const CSharedPointer<const CD
 	if (m_pFilelistStatusBar)
 		m_pFilelistStatusBar->SetHidden(m_pDirectoryListing->GetCount() + 1 - m_indexMapping.size());
 
-	wxASSERT(m_indexMapping.size() == pDirectoryListing->GetCount() + 1);
+	wxASSERT(m_indexMapping.size() <= pDirectoryListing->GetCount() + 1);
 }
 
 void CRemoteListView::UpdateDirectoryListing_Removed(const CSharedPointer<const CDirectoryListing> &pDirectoryListing)
@@ -590,12 +597,12 @@ void CRemoteListView::UpdateDirectoryListing_Removed(const CSharedPointer<const
 			const CDirentry& oldEntry = (*m_pDirectoryListing)[index];
 			if (isSelected)
 			{
-				if (oldEntry.dir)
+				if (oldEntry.is_dir())
 					m_pFilelistStatusBar->UnselectDirectory();
 				else
 					m_pFilelistStatusBar->UnselectFile(oldEntry.size);
 			}
-			if (oldEntry.dir)
+			if (oldEntry.is_dir())
 				m_pFilelistStatusBar->RemoveDirectory();
 			else
 				m_pFilelistStatusBar->RemoveFile(oldEntry.size);
@@ -701,10 +708,7 @@ bool CRemoteListView::UpdateDirectoryListing(const CSharedPointer<const CDirecto
 
 void CRemoteListView::SetDirectoryListing(const CSharedPointer<const CDirectoryListing> &pDirectoryListing, bool modified /*=false*/)
 {
-#ifdef __WXMSW__
-	if (GetEditControl())
-		ListView_CancelEditLabel((HWND)GetHandle());
-#endif
+	CancelLabelEdit();
 
 	bool reset = false;
 	if (!pDirectoryListing || !m_pDirectoryListing)
@@ -723,7 +727,8 @@ void CRemoteListView::SetDirectoryListing(const CSharedPointer<const CDirectoryL
 		if (UpdateDirectoryListing(pDirectoryListing))
 		{
 			wxASSERT(GetItemCount() == (int)m_indexMapping.size());
-			wxASSERT(GetItemCount() == (int)m_fileData.size());
+			wxASSERT(GetItemCount() <= (int)m_fileData.size());
+			wxASSERT(CFilterManager::HasActiveFilters() || GetItemCount() == (int)m_fileData.size());
 			wxASSERT(m_pDirectoryListing->GetCount() + 1 >= (unsigned int)GetItemCount());
 			wxASSERT(m_indexMapping[0] == m_pDirectoryListing->GetCount());
 
@@ -751,7 +756,10 @@ void CRemoteListView::SetDirectoryListing(const CSharedPointer<const CDirectoryL
 	}
 
 	if (m_pFilelistStatusBar)
+	{
 		m_pFilelistStatusBar->UnselectAll();
+		m_pFilelistStatusBar->SetConnected(pDirectoryListing); 
+	}
 
 	m_pDirectoryListing = pDirectoryListing;
 
@@ -779,16 +787,25 @@ void CRemoteListView::SetDirectoryListing(const CSharedPointer<const CDirectoryL
 			const CDirentry& entry = (*m_pDirectoryListing)[i];
 			CGenericFileData data;
 			data.flags = normal;
-			data.icon = entry.dir ? m_dirIcon : -2;
+			if (entry.is_dir())
+			{
+				data.icon = m_dirIcon;
+#ifndef __WXMSW__
+				if (entry.is_link())
+					data.icon += 3;
+#endif
+			}
+			else
+				data.icon = -2;
 			m_fileData.push_back(data);
 
-			if (filter.FilenameFiltered(entry.name, path, entry.dir, entry.size, false, 0))
+			if (filter.FilenameFiltered(entry.name, path, entry.is_dir(), entry.size, false, 0, entry.has_date() ? &entry.time : 0))
 			{
 				hidden++;
 				continue;
 			}
 
-			if (entry.dir)
+			if (entry.is_dir())
 				totalDirCount++;
 			else
 			{
@@ -823,7 +840,7 @@ void CRemoteListView::SetDirectoryListing(const CSharedPointer<const CDirectoryL
 		if (index == -1)
 			resetDropTarget = true;
 		else if (index != (int)m_pDirectoryListing->GetCount())
-			if (!(*m_pDirectoryListing)[index].dir)
+			if (!(*m_pDirectoryListing)[index].is_dir())
 				resetDropTarget = true;
 
 		if (resetDropTarget)
@@ -897,31 +914,31 @@ public:
 		{
 		default:
 		case dirsort_ontop:
-			if (data1.dir)
+			if (data1.is_dir())
 			{
-				if (!data2.dir)
+				if (!data2.is_dir())
 					return -1;
 				else
 					return 0;
 			}
 			else
 			{
-				if (data2.dir)
+				if (data2.is_dir())
 					return 1;
 				else
 					return 0;
 			}
 		case dirsort_onbottom:
-			if (data1.dir)
+			if (data1.is_dir())
 			{
-				if (!data2.dir)
+				if (!data2.is_dir())
 					return 1;
 				else
 					return 0;
 			}
 			else
 			{
-				if (data2.dir)
+				if (data2.is_dir())
 					return -1;
 				else
 					return 0;
@@ -958,16 +975,16 @@ public:
 
 	inline int CmpTime(const CDirentry &data1, const CDirentry &data2) const
 	{
-		if (data1.hasTimestamp == CDirentry::timestamp_none)
+		if (!data1.has_date())
 		{
-			if (data2.hasTimestamp != CDirentry::timestamp_none)
+			if (data2.has_date())
 				return -1;
 			else
 				return 0;
 		}
 		else
 		{
-			if (data2.hasTimestamp == CDirentry::timestamp_none)
+			if (!data2.has_date())
 				return 1;
 
 			if (data1.time < data2.time)
@@ -1059,9 +1076,9 @@ public:
 		CGenericFileData &type1 = m_fileData[a];
 		CGenericFileData &type2 = m_fileData[b];
 		if (type1.fileType.IsEmpty())
-			type1.fileType = m_pRemoteListView->GetType(data1.name, data1.dir);
+			type1.fileType = m_pRemoteListView->GetType(data1.name, data1.is_dir());
 		if (type2.fileType.IsEmpty())
-			type2.fileType = m_pRemoteListView->GetType(data2.name, data2.dir);
+			type2.fileType = m_pRemoteListView->GetType(data2.name, data2.is_dir());
 
 		CMP(CmpStringNoCase, type1.fileType, type2.fileType);
 
@@ -1205,7 +1222,7 @@ void CRemoteListView::OnItemActivated(wxListEvent &event)
 			return;
 		}
 
-		if (entry.dir)
+		if (entry.is_dir())
 		{
 			const int action = COptions::Get()->GetOptionVal(OPTION_DOUBLECLICK_ACTION_DIRECTORY);
 			if (action == 3)
@@ -1217,16 +1234,16 @@ void CRemoteListView::OnItemActivated(wxListEvent &event)
 
 			if (!action)
 			{
-				if (entry.link)
+				if (entry.is_link())
 				{
 					delete m_pLinkResolveState;
 					m_pLinkResolveState = new t_linkResolveState;
 					m_pLinkResolveState->remote_path = m_pDirectoryListing->path;
 					m_pLinkResolveState->link = name;
-					m_pLinkResolveState->local_path = m_pState->GetLocalDir().GetPath();
+					m_pLinkResolveState->local_path = m_pState->GetLocalDir();
 					m_pLinkResolveState->server = *pServer;
 				}
-				m_pState->ChangeRemoteDir(m_pDirectoryListing->path, name, entry.link ? LIST_FLAG_LINK : 0);
+				m_pState->ChangeRemoteDir(m_pDirectoryListing->path, name, entry.is_link() ? LIST_FLAG_LINK : 0);
 			}
 			else
 			{
@@ -1261,7 +1278,7 @@ void CRemoteListView::OnItemActivated(wxListEvent &event)
 				return;
 			}
 
-			m_pQueue->QueueFile(queue_only, true, local_path.GetPath() + name, name, m_pDirectoryListing->path, *pServer, entry.size);
+			m_pQueue->QueueFile(queue_only, true, local_path, CQueueView::ReplaceInvalidCharacters(name), name, m_pDirectoryListing->path, *pServer, entry.size);
 			m_pQueue->QueueFile_Finish(true);
 		}
 	}
@@ -1311,22 +1328,22 @@ void CRemoteListView::OnMenuEnter(wxCommandEvent &event)
 			return;
 		}
 
-		if (!entry.dir)
+		if (!entry.is_dir())
 		{
 			wxBell();
 			return;
 		}
 
-		if (entry.link)
+		if (entry.is_link())
 		{
 			delete m_pLinkResolveState;
 			m_pLinkResolveState = new t_linkResolveState;
 			m_pLinkResolveState->remote_path = m_pDirectoryListing->path;
 			m_pLinkResolveState->link = name;
-			m_pLinkResolveState->local_path = m_pState->GetLocalDir().GetPath();
+			m_pLinkResolveState->local_path = m_pState->GetLocalDir();
 			m_pLinkResolveState->server = *pServer;
 		}
-		m_pState->ChangeRemoteDir(m_pDirectoryListing->path, name, entry.link ? LIST_FLAG_LINK : 0);
+		m_pState->ChangeRemoteDir(m_pDirectoryListing->path, name, entry.is_link() ? LIST_FLAG_LINK : 0);
 	}
 	else
 	{
@@ -1357,6 +1374,7 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 		pMenu->Enable(XRCID("ID_CHMOD"), false);
 		pMenu->Enable(XRCID("ID_EDIT"), false);
 		pMenu->Enable(XRCID("ID_GETURL"), false);
+		pMenu->Enable(XRCID("ID_CONTEXT_REFRESH"), false);
 	}
 	else if ((GetItemCount() && GetItemState(0, wxLIST_STATE_SELECTED)))
 	{
@@ -1396,7 +1414,7 @@ void CRemoteListView::OnContextMenu(wxContextMenuEvent& event)
 				fillCount++;
 				continue;
 			}
-			if ((*m_pDirectoryListing)[index].dir)
+			if ((*m_pDirectoryListing)[index].is_dir())
 				selectedDir = true;
 		}
 		if (!count || fillCount == count)
@@ -1465,7 +1483,7 @@ void CRemoteListView::OnMenuDownload(wxCommandEvent& event)
 			continue;
 		if (m_fileData[index].flags == fill)
 			continue;
-		if ((*m_pDirectoryListing)[index].dir && !idle)
+		if ((*m_pDirectoryListing)[index].is_dir() && !idle)
 		{
 			wxBell();
 			return;
@@ -1509,22 +1527,22 @@ void CRemoteListView::TransferSelectedFiles(const CLocalPath& local_parent, bool
 		const CDirentry& entry = (*m_pDirectoryListing)[index];
 		const wxString& name = entry.name;
 
-		if (entry.dir)
+		if (entry.is_dir())
 		{
 			if (!idle)
 				continue;
 			CLocalPath local_path(local_parent);
-			local_path.AddSegment(name);
+			local_path.AddSegment(CQueueView::ReplaceInvalidCharacters(name));
 			CServerPath remotePath = m_pDirectoryListing->path;
 			if (remotePath.AddSegment(name))
 			{
-				pRecursiveOperation->AddDirectoryToVisit(m_pDirectoryListing->path, name, local_path.GetPath(), entry.link);
+				pRecursiveOperation->AddDirectoryToVisit(m_pDirectoryListing->path, name, local_path, entry.is_link());
 				startRecursive = true;
 			}
 		}
 		else
 		{
-			m_pQueue->QueueFile(queueOnly, true, local_parent.GetPath() + name, name, m_pDirectoryListing->path, *pServer, entry.size);
+			m_pQueue->QueueFile(queueOnly, true, local_parent, CQueueView::ReplaceInvalidCharacters(name), name, m_pDirectoryListing->path, *pServer, entry.size);
 			added = true;
 		}
 	}
@@ -1623,10 +1641,10 @@ void CRemoteListView::OnMenuDelete(wxCommandEvent& event)
 			continue;
 
 		const CDirentry& entry = (*m_pDirectoryListing)[index];
-		if (entry.dir)
+		if (entry.is_dir())
 		{
 			count_dirs++;
-			if (entry.link)
+			if (entry.is_link())
 				selected_link = true;
 		}
 		else
@@ -1685,12 +1703,12 @@ void CRemoteListView::OnMenuDelete(wxCommandEvent& event)
 		const CDirentry& entry = (*m_pDirectoryListing)[index];
 		const wxString& name = entry.name;
 
-		if (entry.dir && (follow_symlink || !entry.link))
+		if (entry.is_dir() && (follow_symlink || !entry.is_link()))
 		{
 			CServerPath remotePath = m_pDirectoryListing->path;
 			if (remotePath.AddSegment(name))
 			{
-				pRecursiveOperation->AddDirectoryToVisit(m_pDirectoryListing->path, name, _T(""), true);
+				pRecursiveOperation->AddDirectoryToVisit(m_pDirectoryListing->path, name, CLocalPath(), true);
 				startRecursive = true;
 			}
 		}
@@ -1743,8 +1761,14 @@ void CRemoteListView::OnMenuRename(wxCommandEvent& event)
 
 void CRemoteListView::OnKeyDown(wxKeyEvent& event)
 {
+#ifdef __WXMAC__
+#define CursorModifierKey wxMOD_CMD
+#else
+#define CursorModifierKey wxMOD_ALT
+#endif
+
 	int code = event.GetKeyCode();
-	if (code == WXK_DELETE)
+	if (code == WXK_DELETE || code == WXK_NUMPAD_DELETE)
 	{
 		if (GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED) == -1)
 		{
@@ -1761,105 +1785,69 @@ void CRemoteListView::OnKeyDown(wxKeyEvent& event)
 		wxCommandEvent tmp;
 		OnMenuRename(tmp);
 	}
-	else if (code == WXK_BACK)
+	else if (code == WXK_RIGHT && event.GetModifiers() == CursorModifierKey)
 	{
-		if (!m_pState->IsRemoteIdle())
-		{
-			wxBell();
-			return;
-		}
-
-		if (!m_pDirectoryListing)
-		{
-			wxBell();
-			return;
-		}
-
-		m_pState->ChangeRemoteDir(m_pDirectoryListing->path, _T(".."));
+		wxListEvent evt;
+		evt.m_itemIndex = GetNextItem(-1, wxLIST_NEXT_ALL, wxLIST_STATE_FOCUSED);
+		OnItemActivated(evt);
+	}
+	else if (code == WXK_DOWN && event.GetModifiers() == CursorModifierKey)
+	{
+		wxCommandEvent cmdEvent;
+		OnMenuDownload(cmdEvent);
 	}
 	else
 		event.Skip();
 }
 
-void CRemoteListView::OnBeginLabelEdit(wxListEvent& event)
+bool CRemoteListView::OnBeginRename(const wxListEvent& event)
 {
 	if (!m_pState->IsRemoteIdle())
 	{
-		event.Veto();
 		wxBell();
-		return;
+		return false;
 	}
 
 	if (!m_pDirectoryListing)
 	{
-		event.Veto();
 		wxBell();
-		return;
+		return false;
 	}
 
 	int item = event.GetIndex();
 	if (!item)
-	{
-		event.Veto();
-		return;
-	}
+		return false;
 
 	int index = GetItemIndex(item);
 	if (index == -1 || m_fileData[index].flags == fill)
-	{
-		event.Veto();
-		return;
-	}
+		return false;
+
+	return true;
 }
 
-void CRemoteListView::OnEndLabelEdit(wxListEvent& event)
+bool CRemoteListView::OnAcceptRename(const wxListEvent& event)
 {
-	int item = event.GetIndex();
-#ifdef __WXMAC__
-	if (item != -1)
-	{
-		int from = item;
-		int to = item;
-		if (from)
-			from--;
-		if (to < GetItemCount() - 1)
-			to++;
-		RefreshItems(from, to);
-	}
-#endif
-
-	if (event.IsEditCancelled() || event.GetLabel() == _T(""))
-	{
-		event.Veto();
-		return;
-	}
-
 	if (!m_pState->IsRemoteIdle())
 	{
-		event.Veto();
 		wxBell();
-		return;
+		return false;
 	}
 
 	if (!m_pDirectoryListing)
 	{
-		event.Veto();
 		wxBell();
-		return;
+		return false;
 	}
 
+	int item = event.GetIndex();
 	if (!item)
-	{
-		event.Veto();
-		return;
-	}
+		return false;
 
 	int index = GetItemIndex(item);
 	if (index == -1 || m_fileData[index].flags == fill)
 	{
 		wxBell();
-		event.Veto();
-		return;
+		return false;
 	}
 
 	const CDirentry& entry = (*m_pDirectoryListing)[index];
@@ -1870,14 +1858,13 @@ void CRemoteListView::OnEndLabelEdit(wxListEvent& event)
 	if (!newPath.ChangePath(newFile, true))
 	{
 		wxMessageBox(_("Filename invalid"), _("Cannot rename file"), wxICON_EXCLAMATION);
-		event.Veto();
-		return;
+		return false;
 	}
 
 	if (newPath == m_pDirectoryListing->path)
 	{
 		if (entry.name == newFile)
-			return;
+			return false;
 
 		// Check if target file already exists
 		for (unsigned int i = 0; i < m_pDirectoryListing->GetCount(); i++)
@@ -1885,10 +1872,7 @@ void CRemoteListView::OnEndLabelEdit(wxListEvent& event)
 			if (newFile == (*m_pDirectoryListing)[i].name)
 			{
 				if (wxMessageBox(_("Target filename already exists, really continue?"), _("File exists"), wxICON_QUESTION | wxYES_NO) != wxYES)
-				{
-					event.Veto();
-					return;
-				}
+					return false;
 
 				break;
 			}
@@ -1896,6 +1880,8 @@ void CRemoteListView::OnEndLabelEdit(wxListEvent& event)
 	}
 
 	m_pState->m_pCommandQueue->ProcessCommand(new CRenameCommand(m_pDirectoryListing->path, entry.name, newPath, newFile));
+
+	return true;
 }
 
 void CRemoteListView::OnMenuChmod(wxCommandEvent& event)
@@ -1930,7 +1916,7 @@ void CRemoteListView::OnMenuChmod(wxCommandEvent& event)
 
 		const CDirentry& entry = (*m_pDirectoryListing)[index];
 
-		if (entry.dir)
+		if (entry.is_dir())
 			dirCount++;
 		else
 			fileCount++;
@@ -2014,17 +2000,17 @@ void CRemoteListView::OnMenuChmod(wxCommandEvent& event)
 		const CDirentry& entry = (*m_pDirectoryListing)[index];
 
 		if (!applyType ||
-			(!entry.dir && applyType == 1) ||
-			(entry.dir && applyType == 2))
+			(!entry.is_dir() && applyType == 1) ||
+			(entry.is_dir() && applyType == 2))
 		{
 			char permissions[9];
 			bool res = pChmodDlg->ConvertPermissions(entry.permissions, permissions);
-			wxString newPerms = pChmodDlg->GetPermissions(res ? permissions : 0, entry.dir);
+			wxString newPerms = pChmodDlg->GetPermissions(res ? permissions : 0, entry.is_dir());
 
 			m_pState->m_pCommandQueue->ProcessCommand(new CChmodCommand(m_pDirectoryListing->path, entry.name, newPerms));
 		}
 
-		if (pChmodDlg->Recursive() && entry.dir)
+		if (pChmodDlg->Recursive() && entry.is_dir())
 			pRecursiveOperation->AddDirectoryToVisit(m_pDirectoryListing->path, entry.name);
 	}
 
@@ -2080,13 +2066,13 @@ void CRemoteListView::ApplyCurrentFilter()
 	for (unsigned int i = 0; i < count; i++)
 	{
 		const CDirentry& entry = (*m_pDirectoryListing)[i];
-		if (filter.FilenameFiltered(entry.name, path, entry.dir, entry.size, false, 0))
+		if (filter.FilenameFiltered(entry.name, path, entry.is_dir(), entry.size, false, 0, entry.has_date() ? &entry.time : 0))
 		{
 			hidden++;
 			continue;
 		}
 
-		if (entry.dir)
+		if (entry.is_dir())
 			totalDirCount++;
 		else
 		{
@@ -2146,7 +2132,7 @@ std::list<wxString> CRemoteListView::RememberSelectedItems(wxString& focused)
 			}
 			const CDirentry& entry = (*m_pDirectoryListing)[index];
 
-			if (entry.dir)
+			if (entry.is_dir())
 				selectedNames.push_back(_T("d") + entry.name);
 			else
 				selectedNames.push_back(_T("-") + entry.name);
@@ -2228,7 +2214,7 @@ void CRemoteListView::ReselectItems(std::list<wxString>& selectedNames, wxString
 				SetItemState(i, wxLIST_STATE_FOCUSED, wxLIST_STATE_FOCUSED);
 				focused = _T("");
 			}
-			if (entry.dir && *iter == (_T("d") + entry.name))
+			if (entry.is_dir() && *iter == (_T("d") + entry.name))
 			{
 				if (firstSelected == -1)
 					firstSelected = i;
@@ -2299,13 +2285,18 @@ void CRemoteListView::RepositionInfoText()
 
 }
 
-void CRemoteListView::OnStateChange(enum t_statechange_notifications notification, const wxString& data)
+void CRemoteListView::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2)
 {
-	wxASSERT(m_pState);
+	wxASSERT(pState);
 	if (notification == STATECHANGE_REMOTE_DIR)
-		SetDirectoryListing(m_pState->GetRemoteDir(), false);
+		SetDirectoryListing(pState->GetRemoteDir(), false);
 	else if (notification == STATECHANGE_REMOTE_DIR_MODIFIED)
-		SetDirectoryListing(m_pState->GetRemoteDir(), true);
+		SetDirectoryListing(pState->GetRemoteDir(), true);
+	else if (notification == STATECHANGE_REMOTE_LINKNOTDIR)
+	{
+		wxASSERT(data2);
+		LinkIsNotDir(*(CServerPath*)data2, data);
+	}
 	else
 	{
 		wxASSERT(notification == STATECHANGE_APPLYFILTER);
@@ -2383,7 +2374,7 @@ void CRemoteListView::OnBeginDrag(wxListEvent& event)
 		int index = GetItemIndex(item);
 		if (index == -1 || m_fileData[index].flags == fill)
 			continue;
-		if ((*m_pDirectoryListing)[index].dir && !idle)
+		if ((*m_pDirectoryListing)[index].is_dir() && !idle)
 		{
 			// Drag could result in recursive operation, don't allow at this point
 			wxBell();
@@ -2425,7 +2416,7 @@ void CRemoteListView::OnBeginDrag(wxListEvent& event)
 			continue;
 		const CDirentry& entry = (*m_pDirectoryListing)[index];
 
-		pRemoteDataObject->AddFile(entry.name, entry.dir, entry.size, entry.link);
+		pRemoteDataObject->AddFile(entry.name, entry.is_dir(), entry.size, entry.is_link());
 	}
 
 	pRemoteDataObject->Finalize();
@@ -2447,6 +2438,8 @@ void CRemoteListView::OnBeginDrag(wxListEvent& event)
 	}
 #endif
 
+	CLabelEditBlocker(*this);
+
 	wxDropSource source(this);
 	source.SetData(object);
 
@@ -2504,7 +2497,7 @@ void CRemoteListView::OnBeginDrag(wxListEvent& event)
 				int index = GetItemIndex(item);
 				if (index == -1 || m_fileData[index].flags == fill)
 					continue;
-				if ((*m_pDirectoryListing)[index].dir && !idle)
+				if ((*m_pDirectoryListing)[index].is_dir() && !idle)
 				{
 					// Drag could result in recursive operation, don't allow at this point
 					wxBell();
@@ -2535,34 +2528,6 @@ void CRemoteListView::OnBeginDrag(wxListEvent& event)
 #endif
 }
 
-bool CRemoteListView::DownloadDroppedFiles(const CRemoteDataObject* pRemoteDataObject, const CLocalPath& path, bool queueOnly)
-{
-	if (!m_pState->IsRemoteIdle())
-		return false;
-
-	wxASSERT(!path.empty());
-
-	CRecursiveOperation* pRecursiveOperation = m_pState->GetRecursiveOperationHandler();
-	wxASSERT(pRecursiveOperation);
-
-	const std::list<CRemoteDataObject::t_fileInfo>& files = pRemoteDataObject->GetFiles();
-	for (std::list<CRemoteDataObject::t_fileInfo>::const_iterator iter = files.begin(); iter != files.end(); iter++)
-	{
-		if (!iter->dir)
-			continue;
-
-		pRecursiveOperation->AddDirectoryToVisit(pRemoteDataObject->GetServerPath(), iter->name, path.GetPath() + iter->name, iter->link);
-	}
-
-	if (IsComparing())
-		ExitComparisonMode();
-
-	CFilterManager filter;
-	pRecursiveOperation->StartRecursiveOperation(queueOnly ? CRecursiveOperation::recursive_addtoqueue : CRecursiveOperation::recursive_download, pRemoteDataObject->GetServerPath(), filter.GetActiveFilters(false));
-
-	return true;
-}
-
 void CRemoteListView::InitDateFormat()
 {
 	const wxString& dateFormat = COptions::Get()->GetOption(OPTION_DATE_FORMAT);
@@ -2607,7 +2572,7 @@ void CRemoteListView::OnMenuEdit(wxCommandEvent& event)
 			continue;
 
 		const CDirentry& entry = (*m_pDirectoryListing)[index];
-		if (entry.dir)
+		if (entry.is_dir())
 		{
 			wxBell();
 			return;
@@ -2744,7 +2709,10 @@ void CRemoteListView::OnMenuEdit(wxCommandEvent& event)
 			continue;
 		}
 
-		m_pQueue->QueueFile(false, true, file, entry.name, path, server, entry.size, CEditHandler::remote);
+		wxString localFile;
+		CLocalPath localPath(file, &localFile);
+
+		m_pQueue->QueueFile(false, true, localPath, localFile, entry.name, path, server, entry.size, CEditHandler::remote);
 		m_pQueue->QueueFile_Finish(true);
 	}
 }
@@ -2847,12 +2815,12 @@ bool CRemoteListView::GetNextFile(wxString& name, bool& dir, wxLongLong& size, w
 	const CDirentry& entry = (*m_pDirectoryListing)[index];
 
 	name = entry.name;
-	dir = entry.dir;
+	dir = entry.is_dir();
 	size = entry.size;
-	if (entry.hasTimestamp != CDirentry::timestamp_none)
+	if (entry.has_date())
 	{
 		date = entry.time;
-		hasTime = entry.hasTimestamp >= CDirentry::timestamp_time;
+		hasTime = entry.has_time();
 	}
 	else
 	{
@@ -2904,9 +2872,6 @@ wxListItemAttr* CRemoteListView::OnGetItemAttr(long item) const
 	return 0;
 }
 
-// Defined in LocalListView.cpp
-extern wxString FormatSize(const wxLongLong& size, bool add_bytes_suffix = false);
-
 // Filenames on VMS systems have a revision suffix, e.g.
 // foo.bar;1
 // foo.bar;2
@@ -2956,10 +2921,10 @@ wxString CRemoteListView::GetItemText(int item, unsigned int column)
 	if (column == 1)
 	{
 		const CDirentry& entry = (*m_pDirectoryListing)[index];
-		if (entry.dir || entry.size < 0)
+		if (entry.is_dir() || entry.size < 0)
 			return _T("");
 		else
-			return FormatSize(entry.size);
+			return CSizeFormat::Format(entry.size);
 	}
 	else if (column == 2)
 	{
@@ -2968,9 +2933,9 @@ wxString CRemoteListView::GetItemText(int item, unsigned int column)
 		{
 			const CDirentry& entry = (*m_pDirectoryListing)[index];
 			if (m_pDirectoryListing->path.GetType() == VMS)
-				data.fileType = GetType(StripVMSRevision(entry.name), entry.dir);
+				data.fileType = GetType(StripVMSRevision(entry.name), entry.is_dir());
 			else
-				data.fileType = GetType(entry.name, entry.dir);
+				data.fileType = GetType(entry.name, entry.is_dir());
 		}
 
 		return data.fileType;
@@ -2978,10 +2943,10 @@ wxString CRemoteListView::GetItemText(int item, unsigned int column)
 	else if (column == 3)
 	{
 		const CDirentry& entry = (*m_pDirectoryListing)[index];
-		if (entry.hasTimestamp == CDirentry::timestamp_none)
+		if (!entry.has_date())
 			return _T("");
 
-		if (entry.hasTimestamp >= CDirentry::timestamp_time)
+		if (entry.has_time())
 			return entry.time.Format(m_timeFormat);
 		else
 			return entry.time.Format(m_dateFormat);
@@ -3038,7 +3003,7 @@ void CRemoteListView::OnExitComparisonMode()
 
 bool CRemoteListView::ItemIsDir(int index) const
 {
-	return (*m_pDirectoryListing)[index].dir;
+	return (*m_pDirectoryListing)[index].is_dir();
 }
 
 wxLongLong CRemoteListView::ItemGetSize(int index) const
@@ -3053,8 +3018,7 @@ void CRemoteListView::LinkIsNotDir(const CServerPath& path, const wxString& link
 
 	if (m_pLinkResolveState->remote_path == path && m_pLinkResolveState->link == link)
 	{
-		wxFileName fn(m_pLinkResolveState->local_path, link);
-		m_pQueue->QueueFile(false, true, fn.GetFullPath(), link, m_pLinkResolveState->remote_path, m_pLinkResolveState->server, -1);
+		m_pQueue->QueueFile(false, true, m_pLinkResolveState->local_path, CQueueView::ReplaceInvalidCharacters(link), link, m_pLinkResolveState->remote_path, m_pLinkResolveState->server, -1);
 		m_pQueue->QueueFile_Finish(true);
 	}
 
@@ -3134,5 +3098,48 @@ void CRemoteListView::OnMenuGeturl(wxCommandEvent& event)
 		wxTheClipboard->SetData(new wxTextDataObject(urls));
 	}
 
+	wxTheClipboard->Flush();
 	wxTheClipboard->Close();
 }
+
+#ifdef __WXMSW__
+int CRemoteListView::GetOverlayIndex(int item)
+{
+	int index = GetItemIndex(item);
+	if (index == -1)
+		return 0;
+	if ((unsigned int)index >= m_pDirectoryListing->GetCount())
+		return 0;
+
+	if ((*m_pDirectoryListing)[index].is_link())
+		return GetLinkOverlayIndex();
+
+	return 0;
+}
+#endif
+
+void CRemoteListView::OnMenuRefresh(wxCommandEvent& event)
+{
+	if (m_pState)
+		m_pState->RefreshRemote();
+}
+
+void CRemoteListView::OnNavigationEvent(bool forward)
+{
+	if (!forward)
+	{
+		if (!m_pState->IsRemoteIdle())
+		{
+			wxBell();
+			return;
+		}
+
+		if (!m_pDirectoryListing)
+		{
+			wxBell();
+			return;
+		}
+
+		m_pState->ChangeRemoteDir(m_pDirectoryListing->path, _T(".."));
+	}
+}
