@@ -1,4 +1,4 @@
-#include "FileZilla.h"
+#include <filezilla.h>
 #include "RemoteTreeView.h"
 #include "commandqueue.h"
 #include <wx/dnd.h>
@@ -8,6 +8,8 @@
 #include "inputdialog.h"
 #include "dragdropmanager.h"
 #include <wx/clipbrd.h>
+#include "queue.h"
+#include "QueueView.h"
 
 #ifdef _DEBUG
 #define new DEBUG_NEW
@@ -109,7 +111,7 @@ public:
 					dir.AddSegment(info.name);
 					if (dir == path || dir.IsParentOf(path, false))
 					{
-						wxMessageBox(_("A directory cannot be dragged into one if its subdirectories."));
+						wxMessageBox(_("A directory cannot be dragged into one of its subdirectories."));
 						return wxDragNone;
 					}
 				}
@@ -208,9 +210,9 @@ protected:
 	wxDataObjectComposite* m_pDataObject;
 };
 
-IMPLEMENT_CLASS(CRemoteTreeView, wxTreeCtrl)
+IMPLEMENT_CLASS(CRemoteTreeView, wxTreeCtrlEx)
 
-BEGIN_EVENT_TABLE(CRemoteTreeView, wxTreeCtrl)
+BEGIN_EVENT_TABLE(CRemoteTreeView, wxTreeCtrlEx)
 EVT_TREE_ITEM_EXPANDING(wxID_ANY, CRemoteTreeView::OnItemExpanding)
 EVT_TREE_SEL_CHANGED(wxID_ANY, CRemoteTreeView::OnSelectionChanged)
 EVT_TREE_ITEM_ACTIVATED(wxID_ANY, CRemoteTreeView::OnItemActivated)
@@ -232,7 +234,7 @@ EVT_MENU(XRCID("ID_GETURL"), CRemoteTreeView::OnMenuGeturl)
 END_EVENT_TABLE()
 
 CRemoteTreeView::CRemoteTreeView(wxWindow* parent, wxWindowID id, CState* pState, CQueueView* pQueue)
-	: wxTreeCtrl(parent, id, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxTR_EDIT_LABELS | wxTR_LINES_AT_ROOT | wxTR_HAS_BUTTONS | wxNO_BORDER | wxTR_HIDE_ROOT),
+	: wxTreeCtrlEx(parent, id, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxTR_EDIT_LABELS | wxTR_LINES_AT_ROOT | wxTR_HAS_BUTTONS | wxNO_BORDER | wxTR_HIDE_ROOT),
 	CSystemImageList(16),
 	CStateEventHandler(pState)
 {
@@ -261,12 +263,12 @@ CRemoteTreeView::~CRemoteTreeView()
 	delete m_pImageList;
 }
 
-void CRemoteTreeView::OnStateChange(enum t_statechange_notifications notification, const wxString& data)
+void CRemoteTreeView::OnStateChange(CState* pState, enum t_statechange_notifications notification, const wxString& data, const void* data2)
 {
 	if (notification == STATECHANGE_REMOTE_DIR)
-		SetDirectoryListing(m_pState->GetRemoteDir(), false);
+		SetDirectoryListing(pState->GetRemoteDir(), false);
 	else if (notification == STATECHANGE_REMOTE_DIR_MODIFIED)
-		SetDirectoryListing(m_pState->GetRemoteDir(), true);
+		SetDirectoryListing(pState->GetRemoteDir(), true);
 	else if (notification == STATECHANGE_APPLYFILTER)
 		ApplyFilters();
 }
@@ -347,7 +349,7 @@ void CRemoteTreeView::SetDirectoryListing(const CSharedPointer<const CDirectoryL
 	m_freezeCount--;
 #endif
 	if (!modified)
-		SelectItem(parent);
+		SafeSelectItem(parent);
 #ifndef __WXMSW__
 	else
 		Refresh();
@@ -497,10 +499,10 @@ bool CRemoteTreeView::HasSubdirs(const CDirectoryListing& listing, const CFilter
 	const wxString path = listing.path.GetPath();
 	for (unsigned int i = 0; i < listing.GetCount(); i++)
 	{
-		if (!listing[i].dir)
+		if (!listing[i].is_dir())
 			continue;
 
-		if (filter.FilenameFiltered(listing[i].name, path, true, -1, false, 0))
+		if (filter.FilenameFiltered(listing[i].name, path, true, -1, false, 0, listing[i].has_date() ? &listing[i].time : 0))
 			continue;
 
 		return true;
@@ -518,10 +520,10 @@ void CRemoteTreeView::DisplayItem(wxTreeItemId parent, const CDirectoryListing&
 	CFilterDialog filter;
 	for (unsigned int i = 0; i < listing.GetCount(); i++)
 	{
-		if (!listing[i].dir)
+		if (!listing[i].is_dir())
 			continue;
 
-		if (filter.FilenameFiltered(listing[i].name, path, true, -1, false, 0))
+		if (filter.FilenameFiltered(listing[i].name, path, true, -1, false, 0, listing[i].has_date() ? &listing[i].time : 0))
 			continue;
 
 		const wxString& name = listing[i].name;
@@ -577,10 +579,10 @@ void CRemoteTreeView::RefreshItem(wxTreeItemId parent, const CDirectoryListing&
 	std::list<wxString> dirs;
 	for (unsigned int i = 0; i < listing.GetCount(); i++)
 	{
-		if (!listing[i].dir)
+		if (!listing[i].is_dir())
 			continue;
 
-		if (!filter.FilenameFiltered(listing[i].name, path, true, -1, false, 0))
+		if (!filter.FilenameFiltered(listing[i].name, path, true, -1, false, 0, listing[i].has_date() ? &listing[i].time : 0))
 			dirs.push_back(listing[i].name);
 	}
 
@@ -1035,7 +1037,7 @@ void CRemoteTreeView::OnMenuChmod(wxCommandEvent& event)
 
 		if (pChmodDlg->Recursive())
 			// Start recursion
-			pRecursiveOperation->AddDirectoryToVisit(path, _T(""), _T(""));
+			pRecursiveOperation->AddDirectoryToVisit(path, _T(""), CLocalPath());
 	}
 	else
 	{
@@ -1089,10 +1091,10 @@ void CRemoteTreeView::OnMenuDownload(wxCommandEvent& event)
 
 	const wxString& name = GetItemText(m_contextMenuItem);
 
-	localDir.AddSegment(name);
+	localDir.AddSegment(CQueueView::ReplaceInvalidCharacters(name));
 
 	CRecursiveOperation* pRecursiveOperation = m_pState->GetRecursiveOperationHandler();
-	pRecursiveOperation->AddDirectoryToVisit(path, _T(""), localDir.GetPath());
+	pRecursiveOperation->AddDirectoryToVisit(path, _T(""), localDir);
 
 	CServerPath currentPath;
 	const wxTreeItemId selected = GetSelection();
@@ -1342,7 +1344,7 @@ void CRemoteTreeView::OnChar(wxKeyEvent& event)
 	wxCommandEvent cmdEvt;
 	if (event.GetKeyCode() == WXK_F2)
 		OnMenuRename(cmdEvt);
-	else if (event.GetKeyCode() == WXK_DELETE)
+	else if (event.GetKeyCode() == WXK_DELETE || event.GetKeyCode() == WXK_NUMPAD_DELETE)
 		OnMenuDelete(cmdEvt);
 	else
 		event.Skip();
@@ -1389,7 +1391,7 @@ void CRemoteTreeView::ApplyFilters()
 				if (path.IsEmpty())
 					continue;
 
-				if (filter.FilenameFiltered(GetItemText(child), path.GetPath(), true, -1, false, 0))
+				if (filter.FilenameFiltered(GetItemText(child), path.GetPath(), true, -1, false, 0, 0))
 				{
 					wxTreeItemId sel = GetSelection();
 					while (sel && sel != child)
@@ -1457,5 +1459,6 @@ void CRemoteTreeView::OnMenuGeturl(wxCommandEvent& event)
 
 	wxTheClipboard->SetData(new wxURLDataObject(url));
 
+	wxTheClipboard->Flush();
 	wxTheClipboard->Close();
 }
