@@ -1,7 +1,9 @@
 #ifndef __FILELIST_STATUSBAR_H__
 #define __FILELIST_STATUSBAR_H__
 
-class CFilelistStatusBar : public wxStatusBar
+#include "option_change_event_handler.h"
+
+class CFilelistStatusBar : public wxStatusBar, protected COptionChangeEventHandler
 {
 public:
 	CFilelistStatusBar(wxWindow* pParent);
@@ -29,6 +31,8 @@ public:
 	void SetConnected(bool connected);
 protected:
 
+	virtual void OnOptionChanged(int option);
+
 	bool m_connected;
 	int m_count_files;
 	int m_count_dirs;
