@@ -147,10 +147,8 @@ private:
 	bool m_insideSetSelection;
 
 #ifdef __WXMSW__
-	static LRESULT CALLBACK WindowProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
-	WNDPROC m_prevWndproc;
-	// char* instead of CFileListCtrl<CFileData> due to bug in mingw compiler
-	static std::map<HWND, char*> m_hwnd_map;
+	virtual WXLRESULT MSWWindowProc(WXUINT nMsg, WXWPARAM wParam, WXLPARAM lParam);
+	virtual bool MSWOnNotify(int idCtrl, WXLPARAM lParam, WXLPARAM *result);
 #else
 	int m_focusItem;
 	std::vector<bool> m_selections;
