@@ -135,7 +135,7 @@ struct t_EngineData;
 class CFileItem : public CQueueItem
 {
 public:
-	CFileItem(CServerItem* parent, bool queued, bool download, const wxString& localFile,
+	CFileItem(CServerItem* parent, bool queued, bool download, const CLocalPath& localPath, const wxString& localFile,
 			const wxString& remoteFile, const CServerPath& remotePath, wxLongLong size);
 	virtual ~CFileItem();
 
@@ -145,17 +145,35 @@ public:
 
 	wxString GetLocalFile() const { return m_localFile; }
 	wxString GetRemoteFile() const { return m_remoteFile; }
-	CServerPath GetRemotePath() const { return m_remotePath; }
+	const CLocalPath& GetLocalPath() const { return m_localPath; }
+	const CServerPath& GetRemotePath() const { return m_remotePath; }
 	wxLongLong GetSize() const { return m_size; }
 	void SetSize(wxLongLong size) { m_size = size; }
-	bool Download() const { return m_download; }
-	bool Queued() const { return m_queued; }
+	inline bool Download() const { return flags & flag_download; }
+	
+	inline bool queued() const { return (flags & flag_queued) != 0; }
+	inline void set_queued(bool q)
+	{
+		if (q)
+			flags |= flag_queued;
+		else
+			flags &= ~flag_queued;
+	}
+
+	inline bool pending_remove() const { return (flags & flag_remove) != 0; }
+	inline void set_pending_remove(bool remove)
+	{
+		if (remove)
+			flags |= flag_remove;
+		else
+			flags &= ~flag_remove;
+	}
 
 	wxString GetIndent() { return m_indent; }
 
 	virtual enum QueueItemType GetType() const { return QueueItemType_File; }
 
-	bool IsActive() const { return m_active; }
+	bool IsActive() const { return (flags & flag_active) != 0; }
 	virtual void SetActive(bool active);
 	
 	virtual void SaveItem(TiXmlElement* pElement) const;
@@ -166,9 +184,7 @@ public:
 	void SetLocalFile(const wxString &file);
 	void SetRemoteFile(const wxString &file);
 
-	bool m_queued;
 	int m_errorCount;
-	bool m_remove;
 	CEditHandler::fileType m_edit;
 
 	wxString m_statusMessage;
@@ -179,25 +195,41 @@ public:
 
 	enum CFileExistsNotification::OverwriteAction m_defaultFileExistsAction;
 
-	bool m_madeProgress;
+	inline bool made_progress() const { return (flags & flag_made_progress) != 0; }
+	inline void set_made_progress(bool made_progress)
+	{
+		if (made_progress)
+			flags |= flag_made_progress;
+		else
+			flags &= ~flag_made_progress;
+	}
 
 	enum CFileExistsNotification::OverwriteAction m_onetime_action;
 
 protected:
+	enum
+	{
+		flag_download = 0x01,
+		flag_active = 0x02,
+		flag_made_progress = 0x04,
+		flag_queued = 0x08,
+		flag_remove = 0x10
+	};
+	int flags;
+
 	enum QueuePriority m_priority;
 
-	bool m_download;
 	wxString m_localFile;
 	wxString m_remoteFile;
-	CServerPath m_remotePath;
+	const CLocalPath m_localPath;
+	const CServerPath m_remotePath;
 	wxLongLong m_size;
-	bool m_active;
 };
 
 class CFolderItem : public CFileItem
 {
 public:
-	CFolderItem(CServerItem* parent, bool queued, const wxString& localFile);
+	CFolderItem(CServerItem* parent, bool queued, const CLocalPath& localPath);
 	CFolderItem(CServerItem* parent, bool queued, const CServerPath& remotePath, const wxString& remoteFile);
 	
 	virtual enum QueueItemType GetType() const { return QueueItemType_Folder; }
@@ -210,20 +242,21 @@ public:
 class CFolderScanItem : public CQueueItem
 {
 public:
-	CFolderScanItem(CServerItem* parent, bool queued, bool download, const wxString& localPath, const CServerPath& remotePath);
+	CFolderScanItem(CServerItem* parent, bool queued, bool download, const CLocalPath& localPath, const CServerPath& remotePath);
 	virtual ~CFolderScanItem() { }
 
 	virtual enum QueueItemType GetType() const { return QueueItemType_FolderScan; }
-	wxString GetLocalPath() const { return m_localPath; }
+	CLocalPath GetLocalPath() const { return m_localPath; }
 	CServerPath GetRemotePath() const { return m_remotePath; }
 	bool Download() const { return m_download; }
-	bool Queued() const { return m_queued; }
 	int GetCount() const { return m_count; }
 	virtual bool TryRemoveAll();
 
+	bool queued() const { return m_queued; }
+	void set_queued(bool q) { m_queued = q; }
+
 	wxString m_statusMessage;
 
-	bool m_queued;
 	volatile bool m_remove;
 	bool m_active;
 
@@ -232,11 +265,12 @@ public:
 	enum CFileExistsNotification::OverwriteAction m_defaultFileExistsAction;
 
 	bool m_dir_is_empty;
-	wxString m_current_local_path;
+	CLocalPath m_current_local_path;
 	CServerPath m_current_remote_path;
 
 protected:
-	wxString m_localPath;
+	bool m_queued;
+	CLocalPath m_localPath;
 	CServerPath m_remotePath;
 	bool m_download;
 };
@@ -271,6 +305,8 @@ public:
 
 	wxString GetTitle() const { return m_title; }
 
+	int GetFileCount() const { return m_fileCount; }
+
 protected:
 
 	// Gets item for given server
