@@ -116,12 +116,12 @@ void CQueueViewFailed::OnRemoveSelected(wxCommandEvent& event)
 			CFileItem* pFileItem = (CFileItem*)pItem;
 			if (pFileItem->m_edit == CEditHandler::remote && pEditHandler)
 			{
-				wxFileName fn(pFileItem->GetLocalFile());
 				if (pFileItem->m_edit == CEditHandler::local)
 				{
-					enum CEditHandler::fileState state = pEditHandler->GetFileState(pFileItem->GetLocalFile());
+					wxString fullPath(pFileItem->GetLocalPath().GetPath() + pFileItem->GetLocalFile());
+					enum CEditHandler::fileState state = pEditHandler->GetFileState(fullPath);
 					if (state == CEditHandler::upload_and_remove_failed)
-						pEditHandler->Remove(pFileItem->GetLocalFile());
+						pEditHandler->Remove(fullPath);
 				}
 				else
 				{
@@ -199,7 +199,7 @@ void CQueueViewFailed::OnRequeueSelected(wxCommandEvent& event)
 				pFileItem->m_errorCount = 0;
 				pFileItem->m_statusMessage.Clear();
 
-				if (!pFileItem->Download() && !wxFileName::FileExists(pFileItem->GetLocalFile()))
+				if (!pFileItem->Download() && !wxFileName::FileExists(pFileItem->GetLocalPath().GetPath() + pFileItem->GetLocalFile()))
 				{
 					failedToRequeueAll = true;
 					RemoveItem(pItem, true, false, false);
@@ -274,7 +274,7 @@ void CQueueViewFailed::OnRequeueSelected(wxCommandEvent& event)
 			pFileItem->m_errorCount = 0;
 			pFileItem->m_statusMessage.Clear();
 
-			if (!pFileItem->Download() && !wxFileName::FileExists(pFileItem->GetLocalFile()))
+			if (!pFileItem->Download() && !wxFileName::FileExists(pFileItem->GetLocalPath().GetPath() + pFileItem->GetLocalFile()))
 			{
 				failedToRequeueAll = true;
 				RemoveItem(pItem, true, false, false);
@@ -372,7 +372,7 @@ void CQueueViewFailed::OnRequeueSelected(wxCommandEvent& event)
 
 void CQueueViewFailed::OnChar(wxKeyEvent& event)
 {
-	if (event.GetKeyCode() == WXK_DELETE)
+	if (event.GetKeyCode() == WXK_DELETE || event.GetKeyCode() == WXK_NUMPAD_DELETE)
 	{
 		wxCommandEvent cmdEvt;
 		OnRemoveSelected(cmdEvt);
