@@ -1,10 +1,11 @@
-#include "FileZilla.h"
+#include <filezilla.h>
 #include "quickconnectbar.h"
 #include "recentserverlist.h"
 #include "commandqueue.h"
 #include "state.h"
 #include "Options.h"
 #include "loginmanager.h"
+#include "Mainfrm.h"
 
 BEGIN_EVENT_TABLE(CQuickconnectBar, wxPanel)
 EVT_BUTTON(XRCID("ID_QUICKCONNECT_OK"), CQuickconnectBar::OnQuickconnect)
@@ -26,10 +27,9 @@ CQuickconnectBar::~CQuickconnectBar()
 {
 }
 
-bool CQuickconnectBar::Create(wxWindow* pParent, CState* pState)
+bool CQuickconnectBar::Create(CMainFrame* pParent)
 {
-	m_pState = pState;
-
+	m_pMainFrame = pParent;
     if (!wxXmlResource::Get()->LoadPanel(this, pParent, _T("ID_QUICKCONNECTBAR")))
 	{
 		wxLogError(_("Cannot load Quickconnect bar from resource file"));
@@ -60,8 +60,9 @@ bool CQuickconnectBar::Create(wxWindow* pParent, CState* pState)
 }
 
 void CQuickconnectBar::OnQuickconnect(wxCommandEvent& event)
-{	
-	if (!m_pState->m_pEngine)
+{
+	CState* pState = CContextManager::Get()->GetCurrentContext();
+	if (!pState || !pState->m_pEngine)
 	{
 		wxMessageBox(_("FTP Engine not initialized, can't connect"), _("FileZilla Error"), wxICON_EXCLAMATION);
 		return;
@@ -72,16 +73,12 @@ void CQuickconnectBar::OnQuickconnect(wxCommandEvent& event)
 	wxString pass = m_pPass->GetValue();
 	wxString port = m_pPort->GetValue();
 	
-	long numericPort = 0;
-	if (port != _T(""))
-		port.ToLong(&numericPort);
-	
 	CServer server;
 
 	wxString error;
 
 	CServerPath path;
-	if (!server.ParseUrl(host, numericPort, user, pass, error, path))
+	if (!server.ParseUrl(host, port, user, pass, error, path))
 	{
 		wxString msg = _("Could not parse server address:");
 		msg += _T("\n");
@@ -118,7 +115,7 @@ void CQuickconnectBar::OnQuickconnect(wxCommandEvent& event)
 	m_pUser->SetValue(server.GetUser());
 	m_pPass->SetValue(server.GetPass());
 
-	if (protocol == HTTP)
+	if (protocol == HTTP || protocol == HTTPS)
 	{
 		wxString error = _("Invalid protocol specified. Valid protocols are:\nftp:// for normal FTP,\nsftp:// for SSH file transfer protocol,\nftps:// for FTP over SSL (implicit) and\nftpes:// for FTP over SSL (explicit).");
 		wxMessageBox(error, _("Syntax error"), wxICON_EXCLAMATION);
@@ -128,7 +125,7 @@ void CQuickconnectBar::OnQuickconnect(wxCommandEvent& event)
 	if (event.GetId() == 1)
 		server.SetBypassProxy(true);
 
-	if (!m_pState->Connect(server, true, path))
+	if (!m_pMainFrame->ConnectToServer(server, path))
 		return;
 
 	CRecentServerList::SetMostRecentServer(server);
@@ -143,16 +140,24 @@ void CQuickconnectBar::OnQuickconnectDropdown(wxCommandEvent& event)
 		pMenu->Append(1, _("Connect bypassing proxy settings"));
 	pMenu->Append(2, _("Clear quickconnect bar"));
 	pMenu->Append(3, _("Clear history"));
-	pMenu->AppendSeparator();
 
 	m_recentServers = CRecentServerList::GetMostRecentServers();
-	unsigned int i = 0;
-	for (std::list<CServer>::const_iterator iter = m_recentServers.begin();
-		iter != m_recentServers.end();
-		iter++, i++)
+	if (!m_recentServers.empty())
 	{
-		pMenu->Append(10 + i, iter->FormatServer());
+		pMenu->AppendSeparator();
+
+		unsigned int i = 0;
+		for (std::list<CServer>::const_iterator iter = m_recentServers.begin();
+			iter != m_recentServers.end();
+			iter++, i++)
+		{
+			wxString name(iter->FormatServer());
+			name.Replace(_T("&"), _T("&&"));
+			pMenu->Append(10 + i, name);
+		}
 	}
+	else
+		pMenu->Enable(3, false);
 
 	XRCCTRL(*this, "ID_QUICKCONNECT_DROPDOWN", wxButton)->PopupMenu(pMenu);
 	delete pMenu;
@@ -186,7 +191,14 @@ void CQuickconnectBar::OnMenu(wxCommandEvent& event)
 			return;
 	}
 
-	m_pState->Connect(server, true);
+	CState* pState = CContextManager::Get()->GetCurrentContext();
+	if (!pState || !pState->m_pEngine)
+	{
+		wxMessageBox(_("FTP Engine not initialized, can't connect"), _("FileZilla Error"), wxICON_EXCLAMATION);
+		return;
+	}
+
+	m_pMainFrame->ConnectToServer(server);
 }
 
 void CQuickconnectBar::ClearFields()
