@@ -1,4 +1,4 @@
-#include "FileZilla.h"
+#include <filezilla.h>
 #include "sitemanager.h"
 #include "Options.h"
 #include "xmlfunctions.h"
@@ -304,10 +304,10 @@ CSiteManager::~CSiteManager()
 	}
 }
 
-bool CSiteManager::Create(wxWindow* parent, const wxString& connected_site_path, const CServer* pServer /*=0*/)
+bool CSiteManager::Create(wxWindow* parent, std::vector<_connected_site> *connected_sites, const CServer* pServer /*=0*/)
 {
 	m_pSiteManagerMutex = new CInterProcessMutex(MUTEX_SITEMANAGERGLOBAL, false);
-	if (!m_pSiteManagerMutex->TryLock())
+	if (m_pSiteManagerMutex->TryLock() == 0)
 	{
 		int answer = wxMessageBox(_("The Site Manager is opened in another instance of FileZilla 3.\nDo you want to continue? Any changes made in the Site Manager won't be saved then."),
 								  _("Site Manager already open"), wxYES_NO | wxICON_QUESTION);
@@ -347,6 +347,29 @@ bool CSiteManager::Create(wxWindow* parent, const wxString& connected_site_path,
 	}
 	int margin = m_pNotebook_Site->GetSize().x - m_pNotebook_Site->GetPage(0)->GetSize().x;
 	m_pNotebook_Site->GetPage(0)->GetSizer()->SetMinSize(wxSize(width - margin, 0));
+#else
+	// Make pages at least wide enough to fit all tabs
+	int width = 10; // Guessed
+	wxClientDC dc(m_pNotebook_Site);
+	for (unsigned int i = 0; i < m_pNotebook_Site->GetPageCount(); i++)
+	{
+		wxCoord w, h;
+		dc.GetTextExtent(m_pNotebook_Site->GetPageText(i), &w, &h);
+		
+		width += w;
+#ifdef __WXMAC__
+		width += 20; // Guessed
+#else
+		width += 10;
+#endif
+	}
+	
+	wxSize page_min_size = m_pNotebook_Site->GetPage(0)->GetSizer()->GetMinSize();
+	if (page_min_size.x < width)
+	{
+		page_min_size.x = width;
+		m_pNotebook_Site->GetPage(0)->GetSizer()->SetMinSize(page_min_size);
+	}
 #endif
 
 	Layout();
@@ -375,6 +398,9 @@ bool CSiteManager::Create(wxWindow* parent, const wxString& connected_site_path,
 	XRCCTRL(*this, "ID_TRANSFERMODE_ACTIVE", wxRadioButton)->Update();
 	XRCCTRL(*this, "ID_TRANSFERMODE_PASSIVE", wxRadioButton)->Update();
 
+	wxTreeItemId item = pTree->GetSelection();
+	if (!item.IsOk())
+		pTree->SelectItem(m_ownSites);
 	SetCtrlState();
 
 	m_pWindowStateManager = new CWindowStateManager(this);
@@ -383,31 +409,43 @@ bool CSiteManager::Create(wxWindow* parent, const wxString& connected_site_path,
 	pTree->SetDropTarget(new CSiteManagerDropTarget(this));
 
 #ifdef __WXGTK__
-	CSiteManagerItemData* data = 0;
-	wxTreeItemId item = pTree->GetSelection();
-	if (item.IsOk())
-		data = reinterpret_cast<CSiteManagerItemData* >(pTree->GetItemData(item));
-	if (!data)
-		XRCCTRL(*this, "wxID_OK", wxButton)->SetFocus();
+	{
+		CSiteManagerItemData* data = 0;
+		wxTreeItemId item = pTree->GetSelection();
+		if (item.IsOk())
+			data = reinterpret_cast<CSiteManagerItemData* >(pTree->GetItemData(item));
+		if (!data)
+			XRCCTRL(*this, "wxID_OK", wxButton)->SetFocus();
+	}
 #endif
 
+	m_connected_sites = connected_sites;
+	MarkConnectedSites();
+
 	if (pServer)
 		CopyAddServer(*pServer);
-	else
-		MarkConnectedSite(connected_site_path);
 
 	return true;
 }
 
-void CSiteManager::MarkConnectedSite(wxString connected_site_path)
+void CSiteManager::MarkConnectedSites()
 {
+	for (int i = 0; i < (int)m_connected_sites->size(); i++)
+		MarkConnectedSite(i);
+}
+
+void CSiteManager::MarkConnectedSite(int connected_site)
+{
+	wxString connected_site_path = (*m_connected_sites)[connected_site].old_path;
+
 	wxTreeCtrl *pTree = XRCCTRL(*this, "ID_SITETREE", wxTreeCtrl);
 	if (!pTree)
 		return;
 
 	if (connected_site_path.Left(1) == _T("1"))
 	{
-		m_changed_bookmark_path = connected_site_path;
+		// Default sites never change
+		(*m_connected_sites)[connected_site].new_path = (*m_connected_sites)[connected_site].old_path;
 		return;
 	}
 
@@ -442,7 +480,8 @@ void CSiteManager::MarkConnectedSite(wxString connected_site_path)
 		return;
 
 	CSiteManagerItemData_Site *site_data = reinterpret_cast<CSiteManagerItemData_Site* >(data);
-	site_data->is_connected_item = true;
+	wxASSERT(site_data->connected_item == -1);
+	site_data->connected_item = connected_site;
 }
 
 void CSiteManager::CreateControls(wxWindow* parent)
@@ -534,12 +573,13 @@ public:
 class CSiteManagerXmlHandler_Tree : public CSiteManagerXmlHandler
 {
 public:
-	CSiteManagerXmlHandler_Tree(wxTreeCtrl* pTree, wxTreeItemId root, const wxString& lastSelection)
-		: m_pTree(pTree), m_item(root)
+	CSiteManagerXmlHandler_Tree(wxTreeCtrl* pTree, wxTreeItemId root, const wxString& lastSelection, bool predefined)
+		: m_pTree(pTree), m_item(root), m_predefined(predefined)
 	{
 		if (!CSiteManager::UnescapeSitePath(lastSelection, m_lastSelection))
 			m_lastSelection.clear();
 		m_wrong_sel_depth = 0;
+		m_kiosk = COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE);
 	}
 
 	virtual ~CSiteManagerXmlHandler_Tree()
@@ -577,6 +617,14 @@ public:
 
 	virtual bool AddSite(CSiteManagerItemData_Site* data)
 	{
+		if (m_kiosk && !m_predefined &&
+			data->m_server.GetLogonType() == NORMAL)
+		{
+			// Clear saved password
+			data->m_server.SetLogonType(ASK);
+			data->m_server.SetUser(data->m_server.GetUser());
+		}
+
 		const wxString name(data->m_server.GetName());
 
 		wxTreeItemId newItem = m_pTree->AppendItem(m_item, name, 2, 2, data);
@@ -649,6 +697,9 @@ protected:
 	int m_wrong_sel_depth;
 
 	std::list<bool> m_expand;
+
+	bool m_predefined;
+	int m_kiosk;
 };
 
 bool CSiteManager::Load()
@@ -698,7 +749,7 @@ bool CSiteManager::Load()
 	}
 	else
 		lastSelection = _T("");
-	CSiteManagerXmlHandler_Tree handler(pTree, treeId, lastSelection);
+	CSiteManagerXmlHandler_Tree handler(pTree, treeId, lastSelection, false);
 
 	bool res = Load(pElement, &handler);
 
@@ -825,7 +876,7 @@ bool CSiteManager::Save(TiXmlElement *pElement /*=0*/, wxTreeItemId treeId /*=wx
 		// to the same file or one is reading while the other one writes.
 		CInterProcessMutex mutex(MUTEX_SITEMANAGER);
 
-		wxFileName file(wxGetApp().GetSettingsDir(), _T("sitemanager.xml"));
+		wxFileName file(COptions::Get()->GetOption(OPTION_DEFAULT_SETTINGSDIR), _T("sitemanager.xml"));
 		CXmlFile xml(file);
 
 		TiXmlElement* pDocument = xml.Load();
@@ -843,7 +894,7 @@ bool CSiteManager::Save(TiXmlElement *pElement /*=0*/, wxTreeItemId treeId /*=wx
 			pDocument->RemoveChild(pServers);
 			pServers = pDocument->FirstChildElement("Servers");
 		}
-		pElement = pDocument->InsertEndChild(TiXmlElement("Servers"))->ToElement();
+		pElement = pDocument->LinkEndChild(new TiXmlElement("Servers"))->ToElement();
 
 		if (!pElement)
 			return true;
@@ -853,7 +904,7 @@ bool CSiteManager::Save(TiXmlElement *pElement /*=0*/, wxTreeItemId treeId /*=wx
 		wxString error;
 		if (!xml.Save(&error))
 		{
-			if (COptions::Get()->GetDefaultVal(DEFAULT_KIOSKMODE) == 2)
+			if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
 				return res;
 			wxString msg = wxString::Format(_("Could not write \"%s\", any changes to the Site Manager could not be saved: %s"), file.GetFullPath().c_str(), error.c_str());
 			wxMessageBox(msg, _("Error writing xml file"), wxICON_ERROR);
@@ -887,18 +938,18 @@ bool CSiteManager::SaveChild(TiXmlElement *pElement, wxTreeItemId child)
 	CSiteManagerItemData* data = reinterpret_cast<CSiteManagerItemData* >(pTree->GetItemData(child));
 	if (!data)
 	{
-		TiXmlNode* pNode = pElement->InsertEndChild(TiXmlElement("Folder"));
+		TiXmlNode* pNode = pElement->LinkEndChild(new TiXmlElement("Folder"));
 		const bool expanded = pTree->IsExpanded(child);
 		SetTextAttribute(pNode->ToElement(), "expanded", expanded ? _T("1") : _T("0"));
 
-		pNode->InsertEndChild(TiXmlText(utf8));
+		pNode->LinkEndChild(new TiXmlText(utf8));
 
 		Save(pNode->ToElement(), child);
 	}
 	else if (data->m_type == CSiteManagerItemData::SITE)
 	{
 		CSiteManagerItemData_Site *site_data = reinterpret_cast<CSiteManagerItemData_Site* >(data);
-		TiXmlElement* pNode = pElement->InsertEndChild(TiXmlElement("Server"))->ToElement();
+		TiXmlElement* pNode = pElement->LinkEndChild(new TiXmlElement("Server"))->ToElement();
 		SetServer(pNode, site_data->m_server);
 
 		// Save comments
@@ -912,19 +963,22 @@ bool CSiteManager::SaveChild(TiXmlElement *pElement, wxTreeItemId child)
 
 		AddTextElementRaw(pNode, "SyncBrowsing", data->m_sync ? "1" : "0");
 
-		pNode->InsertEndChild(TiXmlText(utf8));
+		pNode->LinkEndChild(new TiXmlText(utf8));
 
 		Save(pNode, child);
 
-		if (site_data->is_connected_item)
+		if (site_data->connected_item != -1)
 		{
-			m_changed_bookmark_path = GetSitePath(child);
-			m_changed_bookmark_server = site_data->m_server;
+			if ((*m_connected_sites)[site_data->connected_item].server == site_data->m_server)
+			{
+				(*m_connected_sites)[site_data->connected_item].new_path = GetSitePath(child);
+				(*m_connected_sites)[site_data->connected_item].server = site_data->m_server;
+			}
 		}
 	}
 	else
 	{
-		TiXmlElement* pNode = pElement->InsertEndChild(TiXmlElement("Bookmark"))->ToElement();
+		TiXmlElement* pNode = pElement->LinkEndChild(new TiXmlElement("Bookmark"))->ToElement();
 
 		AddTextElement(pNode, "Name", name);
 
@@ -988,7 +1042,7 @@ bool CSiteManager::Verify()
 		if (host == _T(""))
 		{
 			XRCCTRL(*this, "ID_HOST", wxTextCtrl)->SetFocus();
-			wxMessageBox(_("You have to enter a hostname."));
+			wxMessageBox(_("You have to enter a hostname."), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this); 
 			return false;
 		}
 
@@ -1000,15 +1054,16 @@ bool CSiteManager::Verify()
 			logon_type == ACCOUNT)
 		{
 			XRCCTRL(*this, "ID_LOGONTYPE", wxChoice)->SetFocus();
-			wxMessageBox(_("'Account' logontype not supported by selected protocol"));
+			wxMessageBox(_("'Account' logontype not supported by selected protocol"), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
 			return false;
 		}
 
-		if (COptions::Get()->GetDefaultVal(DEFAULT_KIOSKMODE) != 0 &&
+		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) != 0 &&
+			!IsPredefinedItem(item) &&
 			(logon_type == ACCOUNT || logon_type == NORMAL))
 		{
 			XRCCTRL(*this, "ID_LOGONTYPE", wxChoice)->SetFocus();
-			wxMessageBox(_("FileZilla is running in kiosk mode.\n'Normal' and 'Account' logontypes are not available in this mode."));
+			wxMessageBox(_("FileZilla is running in kiosk mode.\n'Normal' and 'Account' logontypes are not available in this mode."), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
 			return false;
 		}
 
@@ -1019,14 +1074,13 @@ bool CSiteManager::Verify()
 		if (protocol != UNKNOWN)
 			server.SetProtocol(protocol);
 
-		unsigned long port;
-		XRCCTRL(*this, "ID_PORT", wxTextCtrl)->GetValue().ToULong(&port);
+		wxString port = XRCCTRL(*this, "ID_PORT", wxTextCtrl)->GetValue();
 		CServerPath path;
 		wxString error;
 		if (!server.ParseUrl(host, port, _T(""), _T(""), error, path))
 		{
 			XRCCTRL(*this, "ID_HOST", wxTextCtrl)->SetFocus();
-			wxMessageBox(error);
+			wxMessageBox(error, _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
 			return false;
 		}
 
@@ -1043,7 +1097,7 @@ bool CSiteManager::Verify()
 			if (XRCCTRL(*this, "ID_ENCODING", wxTextCtrl)->GetValue() == _T(""))
 			{
 				XRCCTRL(*this, "ID_ENCODING", wxTextCtrl)->SetFocus();
-				wxMessageBox(_("Need to specify a character encoding"));
+				wxMessageBox(_("Need to specify a character encoding"), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
 				return false;
 			}
 		}
@@ -1056,7 +1110,7 @@ bool CSiteManager::Verify()
 			user == _T(""))
 		{
 			XRCCTRL(*this, "ID_USER", wxTextCtrl)->SetFocus();
-			wxMessageBox(_("You have to specify a user name"));
+			wxMessageBox(_("You have to specify a user name"), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
 			return false;
 		}
 
@@ -1075,7 +1129,7 @@ bool CSiteManager::Verify()
 			if (space_only)
 			{
 				XRCCTRL(*this, "ID_USER", wxTextCtrl)->SetFocus();
-				wxMessageBox(_("Username cannot be a series of spaces"));
+				wxMessageBox(_("Username cannot be a series of spaces"), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
 				return false;
 			}
 		}
@@ -1085,7 +1139,7 @@ bool CSiteManager::Verify()
 			XRCCTRL(*this, "ID_ACCOUNT", wxTextCtrl)->GetValue() == _T(""))
 		{
 			XRCCTRL(*this, "ID_ACCOUNT", wxTextCtrl)->SetFocus();
-			wxMessageBox(_("You have to enter an account name"));
+			wxMessageBox(_("You have to enter an account name"), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
 			return false;
 		}
 
@@ -1099,7 +1153,7 @@ bool CSiteManager::Verify()
 			if (!remotePath.SetPath(remotePathRaw))
 			{
 				XRCCTRL(*this, "ID_REMOTEDIR", wxTextCtrl)->SetFocus();
-				wxMessageBox(_("Default remote path cannot be parsed. Make sure it is valid and is supported by the selected servertype."));
+				wxMessageBox(_("Default remote path cannot be parsed. Make sure it is a valid absolute path for the selected server type."), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
 				return false;
 			}
 		}
@@ -1110,7 +1164,7 @@ bool CSiteManager::Verify()
 			if (remotePathRaw.empty() || localPath.empty())
 			{
 				XRCCTRL(*this, "ID_SYNC", wxCheckBox)->SetFocus();
-				wxMessageBox(_("You need to enter both a local and a remote path to enable synchronized browsing for this site."));
+				wxMessageBox(_("You need to enter both a local and a remote path to enable synchronized browsing for this site."), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
 				return false;
 			}
 		}
@@ -1135,7 +1189,7 @@ bool CSiteManager::Verify()
 					msg = wxString::Format(_("Remote path cannot be parsed. Make sure it is a valid absolute path and is supported by the servertype (%s) selected on the parent site."), CServer::GetNameFromServerType(pServer->m_server.GetType()).c_str());
 				else
 					msg = _("Remote path cannot be parsed. Make sure it is a valid absolute path.");
-				wxMessageBox(msg);
+				wxMessageBox(msg, _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
 				return false;
 			}
 		}
@@ -1145,7 +1199,7 @@ bool CSiteManager::Verify()
 		if (remotePathRaw.empty() && localPath.empty())
 		{
 			XRCCTRL(*this, "ID_BOOKMARK_LOCALDIR", wxTextCtrl)->SetFocus();
-			wxMessageBox(_("You need to enter at least one path, empty bookmarks are not supported."));
+			wxMessageBox(_("You need to enter at least one path, empty bookmarks are not supported."), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
 			return false;
 		}
 
@@ -1154,7 +1208,7 @@ bool CSiteManager::Verify()
 			if (remotePathRaw.empty() || localPath.empty())
 			{
 				XRCCTRL(*this, "ID_BOOKMARK_SYNC", wxCheckBox)->SetFocus();
-				wxMessageBox(_("You need to enter both a local and a remote path to enable synchronized browsing for this bookmark."));
+				wxMessageBox(_("You need to enter both a local and a remote path to enable synchronized browsing for this bookmark."), _("Site Manager - Invalid data"), wxICON_EXCLAMATION, this);
 				return false;
 			}
 		}
@@ -1481,10 +1535,10 @@ bool CSiteManager::GetServer(CSiteManagerItemData_Site& data)
 			data.m_localDir = pData->m_localDir;
 		if (!pData->m_remoteDir.IsEmpty())
 			data.m_remoteDir = pData->m_remoteDir;
-		if (!data.m_localDir.empty() && !data.m_remoteDir.IsEmpty())
-			data.m_sync = true;
-		else
+		if (data.m_localDir.empty() || data.m_remoteDir.IsEmpty())
 			data.m_sync = false;
+		else
+			data.m_sync = pData->m_sync;
 	}
 	else
 		data = *(CSiteManagerItemData_Site *)pData;
@@ -1813,7 +1867,7 @@ void CSiteManager::OnCopySite(wxCommandEvent& event)
 	if (data->m_type == CSiteManagerItemData::SITE)
 	{
 		CSiteManagerItemData_Site* newData = new CSiteManagerItemData_Site(*(CSiteManagerItemData_Site *)data);
-		newData->is_connected_item = false;
+		newData->connected_item = -1;
 		newItem = pTree->AppendItem(parent, newName, 2, 2, newData);
 
 		wxTreeItemIdValue cookie;
@@ -1875,7 +1929,7 @@ bool CSiteManager::LoadDefaultSites()
 	}
 	else
 		lastSelection = _T("");
-	CSiteManagerXmlHandler_Tree handler(pTree, m_predefinedSites, lastSelection);
+	CSiteManagerXmlHandler_Tree handler(pTree, m_predefinedSites, lastSelection, true);
 
 	Load(pElement, &handler);
 
@@ -2288,7 +2342,7 @@ bool CSiteManager::MoveItems(wxTreeItemId source, wxTreeItemId target, bool copy
 		else if (data->m_type == CSiteManagerItemData::SITE)
 		{
 			CSiteManagerItemData_Site* newData = new CSiteManagerItemData_Site(*(CSiteManagerItemData_Site *)data);
-			newData->is_connected_item = false;
+			newData->connected_item = -1;
 			pTree->SetItemData(newItem, newData);
 		}
 		else
@@ -2397,7 +2451,7 @@ void CSiteManager::AddNewSite(wxTreeItemId parent, const CServer& server, bool c
 
 	CSiteManagerItemData_Site* pData = new CSiteManagerItemData_Site(server);
 	if (connected)
-		pData->is_connected_item = true;
+		pData->connected_item = 0;
 
 	wxTreeItemId newItem = pTree->AppendItem(parent, name, 2, 2, pData);
 	pTree->SortChildren(parent);
@@ -2633,7 +2687,7 @@ void CSiteManager::OnExportSelected(wxCommandEvent& event)
 
 	TiXmlElement* exportRoot = xml.CreateEmpty();
 
-	TiXmlElement* pServers = exportRoot->InsertEndChild(TiXmlElement("Servers"))->ToElement();
+	TiXmlElement* pServers = exportRoot->LinkEndChild(new TiXmlElement("Servers"))->ToElement();
 	SaveChild(pServers, m_contextMenuItem);
 
 	wxString error;
@@ -2858,20 +2912,20 @@ wxString CSiteManager::AddServer(CServer server)
 
 	server.SetName(name);
 
-	TiXmlElement* pServer = pElement->InsertEndChild(TiXmlElement("Server"))->ToElement();
+	TiXmlElement* pServer = pElement->LinkEndChild(new TiXmlElement("Server"))->ToElement();
 	SetServer(pServer, server);
 
 	char* utf8 = ConvUTF8(name);
 	if (utf8)
 	{
-		pServer->InsertEndChild(TiXmlText(utf8));
+		pServer->LinkEndChild(new TiXmlText(utf8));
 		delete [] utf8;
 	}
 
 	wxString error;
 	if (!file.Save(&error))
 	{
-		if (COptions::Get()->GetDefaultVal(DEFAULT_KIOSKMODE) == 2)
+		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
 			return _T("");
 
 		wxString msg = wxString::Format(_("Could not write \"%s\", any changes to the Site Manager could not be saved: %s"), file.GetFileName().GetFullPath().c_str(), error.c_str());
@@ -2986,7 +3040,7 @@ bool CSiteManager::AddBookmark(wxString sitePath, const wxString& name, const wx
 	if (pInsertBefore)
 		pBookmark = pChild->InsertBeforeChild(pInsertBefore, TiXmlElement("Bookmark"))->ToElement();
 	else
-		pBookmark = pChild->InsertEndChild(TiXmlElement("Bookmark"))->ToElement();
+		pBookmark = pChild->LinkEndChild(new TiXmlElement("Bookmark"))->ToElement();
 	AddTextElement(pBookmark, "Name", name);
 	if (!local_dir.empty())
 		AddTextElement(pBookmark, "LocalDir", local_dir);
@@ -2998,7 +3052,7 @@ bool CSiteManager::AddBookmark(wxString sitePath, const wxString& name, const wx
 	wxString error;
 	if (!file.Save(&error))
 	{
-		if (COptions::Get()->GetDefaultVal(DEFAULT_KIOSKMODE) == 2)
+		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
 			return true;
 
 		wxString msg = wxString::Format(_("Could not write \"%s\", the selected sites could not be exported: %s"), file.GetFileName().GetFullPath().c_str(), error.c_str());
@@ -3060,7 +3114,7 @@ bool CSiteManager::ClearBookmarks(wxString sitePath)
 	wxString error;
 	if (!file.Save(&error))
 	{
-		if (COptions::Get()->GetDefaultVal(DEFAULT_KIOSKMODE) == 2)
+		if (COptions::Get()->GetOptionVal(OPTION_DEFAULT_KIOSKMODE) == 2)
 			return true;
 
 		wxString msg = wxString::Format(_("Could not write \"%s\", the selected sites could not be exported: %s"), file.GetFileName().GetFullPath().c_str(), error.c_str());
@@ -3069,11 +3123,3 @@ bool CSiteManager::ClearBookmarks(wxString sitePath)
 
 	return true;
 }
-
-wxString CSiteManager::GetChangedBookmarkPath(const CServer* pServer)
-{
-	if (pServer && m_changed_bookmark_server != *pServer)
-		return wxEmptyString;
-	
-	return m_changed_bookmark_path;
-}
