@@ -1,5 +1,6 @@
-#include "FileZilla.h"
-#include "directorylistingparser.h"
+#include <libfilezilla.h>
+#include <directorylistingparser.h>
+
 #include <cppunit/extensions/HelperMacros.h>
 #include <list>
 
@@ -77,12 +78,9 @@ void CDirectoryListingParserTest::InitEntries()
 				512,
 				_T("dr-xr-xr-x"),
 				_T("root other"),
-				true,
-				false,
+				CDirentry::flag_dir | CDirentry::flag_timestamp_date,
 				_T(""),
-				CDirentry::timestamp_date,
-				wxDateTime(8, wxDateTime::Apr, 1994),
-				false
+				wxDateTime(8, wxDateTime::Apr, 1994)
 			},
 			DEFAULT
 		});
@@ -95,12 +93,9 @@ void CDirectoryListingParserTest::InitEntries()
 				531,
 				_T("-rw-r--r--"),
 				_T("root other"),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
 				_T(""),
-				CDirentry::timestamp_time,
-				wxDateTime(29, wxDateTime::Mar, calcYear(3, 29), 3, 26),
-				false
+				wxDateTime(29, wxDateTime::Mar, calcYear(3, 29), 3, 26)
 			},
 			DEFAULT
 		});
@@ -113,12 +108,9 @@ void CDirectoryListingParserTest::InitEntries()
 				512,
 				_T("dr-xr-xr-x"),
 				_T("root"),
-				true,
-				false,
+				CDirentry::flag_dir | CDirentry::flag_timestamp_date,
 				_T(""),
-				CDirentry::timestamp_date,
-				wxDateTime(8, wxDateTime::Apr, 1994),
-				false
+				wxDateTime(8, wxDateTime::Apr, 1994)
 			},
 			DEFAULT
 		});
@@ -131,12 +123,9 @@ void CDirectoryListingParserTest::InitEntries()
 				7,
 				_T("lrwxrwxrwx"),
 				_T("root other"),
-				true,
-				true,
+				CDirentry::flag_dir | CDirentry::flag_link | CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
 				_T("usr/bin"),
-				CDirentry::timestamp_time,
-				wxDateTime(25, wxDateTime::Jan, calcYear(1, 25), 0, 17),
-				false
+				wxDateTime(25, wxDateTime::Jan, calcYear(1, 25), 0, 17)
 			},
 			DEFAULT
 		});
@@ -151,12 +140,9 @@ void CDirectoryListingParserTest::InitEntries()
 				531,
 				_T("-rw-r--r--"),
 				_T("root other"),
-				false,
-				false,
+				CDirentry::flag_timestamp_date,
 				_T(""),
-				CDirentry::timestamp_date,
-				wxDateTime(26, wxDateTime::Sep, 2000),
-				false
+				wxDateTime(26, wxDateTime::Sep, 2000)
 			},
 			DEFAULT
 		});
@@ -168,12 +154,9 @@ void CDirectoryListingParserTest::InitEntries()
 				531,
 				_T("-rw-r--r--"),
 				_T("root other"),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
 				_T(""),
-				CDirentry::timestamp_time,
-				wxDateTime(26, wxDateTime::Sep, calcYear(9, 26), 13, 45),
-				false
+				wxDateTime(26, wxDateTime::Sep, calcYear(9, 26), 13, 45)
 			},
 			DEFAULT
 		});
@@ -185,12 +168,9 @@ void CDirectoryListingParserTest::InitEntries()
 				531,
 				_T("-rw-r--r--"),
 				_T("root other"),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
 				_T(""),
-				CDirentry::timestamp_time,
-				wxDateTime(7, wxDateTime::Jun, 2005, 21, 22),
-				false
+				wxDateTime(7, wxDateTime::Jun, 2005, 21, 22)
 			},
 			DEFAULT
 		});
@@ -204,12 +184,9 @@ void CDirectoryListingParserTest::InitEntries()
 				335 * 1024 / 10,
 				_T("-rw-r--r--"),
 				_T("root other"),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
 				_T(""),
-				CDirentry::timestamp_time,
-				wxDateTime(5, wxDateTime::Oct, calcYear(10, 5), 21, 22),
-				false
+				wxDateTime(5, wxDateTime::Oct, calcYear(10, 5), 21, 22)
 			},
 			DEFAULT
 		});
@@ -224,12 +201,9 @@ void CDirectoryListingParserTest::InitEntries()
 				512,
 				_T("d [R----F--]"),
 				_T("supervisor"),
-				true,
-				false,
+				CDirentry::flag_dir | CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
 				_T(""),
-				CDirentry::timestamp_time,
-				wxDateTime(16, wxDateTime::Jan, calcYear(1, 16), 18, 53),
-				false
+				wxDateTime(16, wxDateTime::Jan, calcYear(1, 16), 18, 53)
 			},
 			DEFAULT
 		});
@@ -241,12 +215,9 @@ void CDirectoryListingParserTest::InitEntries()
 				214059,
 				_T("- [R----F--]"),
 				_T("rhesus"),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
 				_T(""),
-				CDirentry::timestamp_time,
-				wxDateTime(20, wxDateTime::Oct, calcYear(10, 20), 15, 27),
-				false
+				wxDateTime(20, wxDateTime::Oct, calcYear(10, 20), 15, 27)
 			},
 			DEFAULT
 		});
@@ -265,12 +236,9 @@ void CDirectoryListingParserTest::InitEntries()
 				1392298,
 				_T("-------r--"),
 				_T("1391972"),
-				false,
-				false,
+				CDirentry::flag_timestamp_date,
 				_T(""),
-				CDirentry::timestamp_date,
-				wxDateTime(22, wxDateTime::Nov, 1995),
-				false
+				wxDateTime(22, wxDateTime::Nov, 1995)
 			},
 			DEFAULT
 		});
@@ -282,12 +250,9 @@ void CDirectoryListingParserTest::InitEntries()
 				2,
 				_T("drwxrwxr-x"),
 				_T("folder"),
-				true,
-				false,
+				CDirentry::flag_dir | CDirentry::flag_timestamp_date,
 				_T(""),
-				CDirentry::timestamp_date,
-				wxDateTime(10, wxDateTime::May, 1996),
-				false
+				wxDateTime(10, wxDateTime::May, 1996)
 			},
 			DEFAULT
 		});
@@ -300,12 +265,9 @@ void CDirectoryListingParserTest::InitEntries()
 				531,
 				_T("-rw-r--r--"),
 				_T("group domain user"),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
 				_T(""),
-				CDirentry::timestamp_time,
-				wxDateTime(29, wxDateTime::Jan, calcYear(1, 29), 3, 26),
-				false
+				wxDateTime(29, wxDateTime::Jan, calcYear(1, 29), 3, 26)
 			},
 			DEFAULT
 		});
@@ -324,12 +286,9 @@ void CDirectoryListingParserTest::InitEntries()
 				280,
 				_T("755"),
 				_T(""),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
 				_T(""),
-				CDirentry::timestamp_seconds,
-				utc,
-				false
+				utc
 			},
 			DEFAULT
 		});
@@ -343,12 +302,9 @@ void CDirectoryListingParserTest::InitEntries()
 				-1,
 				_T(""),
 				_T(""),
-				true,
-				false,
+				CDirentry::flag_dir | CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
 				_T(""),
-				CDirentry::timestamp_seconds,
-				utc,
-				false
+				utc
 			},
 			DEFAULT
 		});
@@ -363,12 +319,9 @@ void CDirectoryListingParserTest::InitEntries()
 				-1,
 				_T(""),
 				_T(""),
-				true,
-				false,
+				CDirentry::flag_dir | CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
 				_T(""),
-				CDirentry::timestamp_time,
-				wxDateTime(27, wxDateTime::Apr, 2000, 12, 9),
-				false
+				wxDateTime(27, wxDateTime::Apr, 2000, 12, 9)
 			},
 			DEFAULT
 		});
@@ -381,12 +334,9 @@ void CDirectoryListingParserTest::InitEntries()
 				589,
 				_T(""),
 				_T(""),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
 				_T(""),
-				CDirentry::timestamp_time,
-				wxDateTime(6, wxDateTime::Apr, 2000, 15, 47),
-				false
+				wxDateTime(6, wxDateTime::Apr, 2000, 15, 47)
 			},
 			DEFAULT
 		});
@@ -398,12 +348,9 @@ void CDirectoryListingParserTest::InitEntries()
 				-1,
 				_T(""),
 				_T(""),
-				true,
-				false,
+				CDirentry::flag_dir | CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
 				_T(""),
-				CDirentry::timestamp_time,
-				wxDateTime(2, wxDateTime::Sep, 2002, 18, 48),
-				false
+				wxDateTime(2, wxDateTime::Sep, 2002, 18, 48)
 			},
 			DEFAULT
 		});
@@ -415,12 +362,9 @@ void CDirectoryListingParserTest::InitEntries()
 				9730,
 				_T(""),
 				_T(""),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
 				_T(""),
-				CDirentry::timestamp_time,
-				wxDateTime(2, wxDateTime::Sep, 2002, 19, 6),
-				false
+				wxDateTime(2, wxDateTime::Sep, 2002, 19, 6)
 			},
 			DEFAULT
 		});
@@ -435,12 +379,9 @@ void CDirectoryListingParserTest::InitEntries()
 				12345,
 				_T("0100644"),
 				_T("500 101"),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
 				_T(""),
-				CDirentry::timestamp_seconds,
-				utc,
-				false
+				utc
 			},
 			DEFAULT
 		});
@@ -455,12 +396,9 @@ void CDirectoryListingParserTest::InitEntries()
 				206876,
 				_T(""),
 				_T(""),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
 				_T(""),
-				CDirentry::timestamp_time,
-				wxDateTime(4, wxDateTime::Apr, 2000, 21, 6),
-				false
+				wxDateTime(4, wxDateTime::Apr, 2000, 21, 6)
 			},
 			DEFAULT
 		});
@@ -472,12 +410,9 @@ void CDirectoryListingParserTest::InitEntries()
 				0,
 				_T(""),
 				_T(""),
-				true,
-				false,
+				CDirentry::flag_dir | CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
 				_T(""),
-				CDirentry::timestamp_time,
-				wxDateTime(12, wxDateTime::Dec, 2002, 2, 13),
-				false
+				wxDateTime(12, wxDateTime::Dec, 2002, 2, 13)
 			},
 			DEFAULT
 		});
@@ -491,12 +426,9 @@ void CDirectoryListingParserTest::InitEntries()
 				9,
 				_T("-rwxr-xr-x"),
 				_T("user group"),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
 				_T(""),
-				CDirentry::timestamp_time,
-				wxDateTime(8, wxDateTime::Oct, 2002, 9, 47),
-				false
+				wxDateTime(8, wxDateTime::Oct, 2002, 9, 47)
 			},
 			DEFAULT
 		});
@@ -512,12 +444,9 @@ void CDirectoryListingParserTest::InitEntries()
 				36611,
 				_T(""),
 				_T(""),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
 				_T(""),
-				CDirentry::timestamp_time,
-				wxDateTime(23, wxDateTime::Apr, 2003, 10, 57),
-				false
+				wxDateTime(23, wxDateTime::Apr, 2003, 10, 57)
 			},
 			DEFAULT
 		});
@@ -529,12 +458,9 @@ void CDirectoryListingParserTest::InitEntries()
 				1123,
 				_T(""),
 				_T(""),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
 				_T(""),
-				CDirentry::timestamp_time,
-				wxDateTime(14, wxDateTime::Jul, 1999, 12, 37),
-				false
+				wxDateTime(14, wxDateTime::Jul, 1999, 12, 37)
 			},
 			DEFAULT
 		});
@@ -547,12 +473,9 @@ void CDirectoryListingParserTest::InitEntries()
 				0,
 				_T(""),
 				_T(""),
-				true,
-				false,
+				CDirentry::flag_dir | CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
 				_T(""),
-				CDirentry::timestamp_time,
-				wxDateTime(11, wxDateTime::Feb, 2003, 16, 15),
-				false
+				wxDateTime(11, wxDateTime::Feb, 2003, 16, 15)
 			},
 			DEFAULT
 		});
@@ -565,12 +488,9 @@ void CDirectoryListingParserTest::InitEntries()
 				1123,
 				_T(""),
 				_T(""),
-				true,
-				false,
+				CDirentry::flag_dir | CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
 				_T(""),
-				CDirentry::timestamp_time,
-				wxDateTime(5, wxDateTime::Oct, 2000, 23, 38),
-				false
+				wxDateTime(5, wxDateTime::Oct, 2000, 23, 38)
 			},
 			DEFAULT
 		});
@@ -585,12 +505,9 @@ void CDirectoryListingParserTest::InitEntries()
 				2235,
 				_T("dr-xr-xr-x"),
 				_T("root other"),
-				true,
-				false,
+				CDirentry::flag_dir | CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
 				_T(""),
-				CDirentry::timestamp_time,
-				wxDateTime(26, wxDateTime::Jul, calcYear(7, 26), 20, 10),
-				false
+				wxDateTime(26, wxDateTime::Jul, calcYear(7, 26), 20, 10)
 			},
 			DEFAULT
 		});
@@ -602,12 +519,9 @@ void CDirectoryListingParserTest::InitEntries()
 				2235,
 				_T("-r-xr-xr-x"),
 				_T("root other"),
-				false,
-				false,
+				CDirentry::flag_timestamp_date,
 				_T(""),
-				CDirentry::timestamp_date,
-				wxDateTime(2, wxDateTime::Oct, 2003),
-				false
+				wxDateTime(2, wxDateTime::Oct, 2003)
 			},
 			DEFAULT
 		});
@@ -619,12 +533,9 @@ void CDirectoryListingParserTest::InitEntries()
 				2235,
 				_T("-r-xr-xr-x"),
 				_T("root other"),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
 				_T(""),
-				CDirentry::timestamp_time,
-				wxDateTime(12, wxDateTime::Oct, 1999, 17, 12),
-				false
+				wxDateTime(12, wxDateTime::Oct, 1999, 17, 12)
 			},
 			DEFAULT
 		});
@@ -636,12 +547,9 @@ void CDirectoryListingParserTest::InitEntries()
 				2235,
 				_T("-r-xr-xr-x"),
 				_T("root other"),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
 				_T(""),
-				CDirentry::timestamp_time,
-				wxDateTime(24, wxDateTime::Apr, 2003, 17, 12),
-				false
+				wxDateTime(24, wxDateTime::Apr, 2003, 17, 12)
 			},
 			DEFAULT
 		});
@@ -656,12 +564,9 @@ void CDirectoryListingParserTest::InitEntries()
 				8473,
 				_T("-rw-r--r--"),
 				_T("root sys"),
-				false,
-				false,
+				CDirentry::flag_timestamp_date,
 				_T(""),
-				CDirentry::timestamp_date,
-				wxDateTime(18, wxDateTime::Apr, 2003),
-				false
+				wxDateTime(18, wxDateTime::Apr, 2003)
 			},
 			DEFAULT
 		});
@@ -675,12 +580,9 @@ void CDirectoryListingParserTest::InitEntries()
 				0,
 				_T("-rwxrwxrwx"),
 				_T("root staff"),
-				false,
-				false,
+				CDirentry::flag_timestamp_date,
 				_T(""),
-				CDirentry::timestamp_date,
-				wxDateTime(20, wxDateTime::Mar, 2003),
-				false
+				wxDateTime(20, wxDateTime::Mar, 2003)
 			},
 			DEFAULT
 		});
@@ -692,12 +594,9 @@ void CDirectoryListingParserTest::InitEntries()
 				2096,
 				_T("-r--r--r--"),
 				_T("root root"),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
 				_T(""),
-				CDirentry::timestamp_time,
-				wxDateTime(17, wxDateTime::Aug, calcYear(8, 17), 8, 52),
-				false
+				wxDateTime(17, wxDateTime::Aug, calcYear(8, 17), 8, 52)
 			},
 			DEFAULT
 		});
@@ -709,12 +608,9 @@ void CDirectoryListingParserTest::InitEntries()
 				96,
 				_T("-r-xr-xr-x"),
 				_T("root root"),
-				false,
-				false,
+				CDirentry::flag_timestamp_date,
 				_T(""),
-				CDirentry::timestamp_date,
-				wxDateTime(15, wxDateTime::Jul, 2004),
-				false
+				wxDateTime(15, wxDateTime::Jul, 2004)
 			},
 			DEFAULT
 		});
@@ -729,12 +625,9 @@ void CDirectoryListingParserTest::InitEntries()
 				512,
 				_T("RWE,RWE,RE,RE"),
 				_T("root,root"),
-				true,
-				false,
+				CDirentry::flag_dir | CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
 				_T(""),
-				CDirentry::timestamp_time,
-				wxDateTime(19, wxDateTime::Nov, 2001, 21, 41),
-				false
+				wxDateTime(19, wxDateTime::Nov, 2001, 21, 41)
 			},
 			DEFAULT
 		});
@@ -747,12 +640,9 @@ void CDirectoryListingParserTest::InitEntries()
 				79360,
 				_T(""),
 				_T(""),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
 				_T(""),
-				CDirentry::timestamp_seconds,
-				wxDateTime(2, wxDateTime::Jul, 2003, 10, 30, 13),
-				false
+				wxDateTime(2, wxDateTime::Jul, 2003, 10, 30, 13)
 			},
 			DEFAULT
 		});
@@ -765,12 +655,9 @@ void CDirectoryListingParserTest::InitEntries()
 				1024,
 				_T("RWED,RWED,RE,"),
 				_T("IV2_XXX"),
-				false,
-				false,
+				CDirentry::flag_timestamp_date,
 				_T(""),
-				CDirentry::timestamp_date,
-				wxDateTime(7, wxDateTime::Jan, 2000),
-				false
+				wxDateTime(7, wxDateTime::Jan, 2000)
 			},
 			DEFAULT
 		});
@@ -783,12 +670,9 @@ void CDirectoryListingParserTest::InitEntries()
 				3072,
 				_T("RWED,RWED,RE,"),
 				_T("PRONAS"),
-				false,
-				false,
+				CDirentry::flag_timestamp_date,
 				_T(""),
-				CDirentry::timestamp_date,
-				wxDateTime(15, wxDateTime::Jul, 2002),
-				false
+				wxDateTime(15, wxDateTime::Jul, 2002)
 			},
 			DEFAULT
 		});
@@ -800,12 +684,9 @@ void CDirectoryListingParserTest::InitEntries()
 				87436288,
 				_T("RWED,RWED,RE,"),
 				_T("FTP_CLIENT,SCOT"),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
 				_T(""),
-				CDirentry::timestamp_seconds,
-				wxDateTime(24, wxDateTime::Apr, 2003, 8, 16, 15),
-				false
+				wxDateTime(24, wxDateTime::Apr, 2003, 8, 16, 15)
 			},
 			DEFAULT
 		});
@@ -817,12 +698,9 @@ void CDirectoryListingParserTest::InitEntries()
 				5120,
 				_T("RWED,RWED,RE,"),
 				_T("FTP_CLIENT,SCOT"),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
 				_T(""),
-				CDirentry::timestamp_seconds,
-				wxDateTime(2, wxDateTime::Jul, 2003, 10, 30, 8),
-				false
+				wxDateTime(2, wxDateTime::Jul, 2003, 10, 30, 8)
 			},
 			DEFAULT
 		});
@@ -835,12 +713,9 @@ void CDirectoryListingParserTest::InitEntries()
 				512,
 				_T("RWE,RWE,RE,"),
 				_T("SUMMARY"),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
 				_T(""),
-				CDirentry::timestamp_time,
-				wxDateTime(2, wxDateTime::Aug, 2006, 13, 5),
-				false
+				wxDateTime(2, wxDateTime::Aug, 2006, 13, 5)
 			},
 			DEFAULT
 		});
@@ -852,12 +727,9 @@ void CDirectoryListingParserTest::InitEntries()
 				3229696,
 				_T("RWED,RWED,R,"),
 				_T(""),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
 				_T(""),
-				CDirentry::timestamp_seconds,
-				wxDateTime(17, wxDateTime::Jun, 1994, 17, 25, 37),
-				false
+				wxDateTime(17, wxDateTime::Jun, 1994, 17, 25, 37)
 			},
 			DEFAULT
 		});
@@ -873,12 +745,9 @@ void CDirectoryListingParserTest::InitEntries()
 				77824,
 				_T(""),
 				_T("QSYS"),
-				true,
-				false,
+				CDirentry::flag_dir | CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
 				_T(""),
-				CDirentry::timestamp_seconds,
-				wxDateTime(23, wxDateTime::Feb, 2000, 15, 9, 55),
-				false
+				wxDateTime(23, wxDateTime::Feb, 2000, 15, 9, 55)
 			},
 			DEFAULT
 		});
@@ -890,12 +759,9 @@ void CDirectoryListingParserTest::InitEntries()
 				77824,
 				_T(""),
 				_T("QSYS"),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
 				_T(""),
-				CDirentry::timestamp_seconds,
-				wxDateTime(23, wxDateTime::Feb, 2000, 15, 9, 55),
-				false
+				wxDateTime(23, wxDateTime::Feb, 2000, 15, 9, 55)
 			},
 			DEFAULT
 		});
@@ -908,12 +774,9 @@ void CDirectoryListingParserTest::InitEntries()
 				123456,
 				_T("-r-xr-xr-x"),
 				_T("longowner longgroup"),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
 				_T(""),
-				CDirentry::timestamp_time,
-				wxDateTime(12, wxDateTime::Feb, calcYear(2, 12), 17, 20),
-				false
+				wxDateTime(12, wxDateTime::Feb, calcYear(2, 12), 17, 20)
 			},
 			DEFAULT
 		});
@@ -926,12 +789,9 @@ void CDirectoryListingParserTest::InitEntries()
 				4512,
 				_T("-r-xr-xr-x"),
 				_T("owner group"),
-				false,
-				false,
+				CDirentry::flag_timestamp_date,
 				_T(""),
-				CDirentry::timestamp_date,
-				wxDateTime(1, wxDateTime::Jun, 1999),
-				false
+				wxDateTime(1, wxDateTime::Jun, 1999)
 			},
 			DEFAULT
 		});
@@ -944,12 +804,9 @@ void CDirectoryListingParserTest::InitEntries()
 				1014196,
 				_T(""),
 				_T(""),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
 				_T(""),
-				CDirentry::timestamp_seconds,
-				wxDateTime(3, wxDateTime::Jun, 2004, 10, 20, 3),
-				false
+				wxDateTime(3, wxDateTime::Jun, 2004, 10, 20, 3)
 			},
 			DEFAULT
 		});
@@ -962,12 +819,9 @@ void CDirectoryListingParserTest::InitEntries()
 				2048,
 				_T(""),
 				_T(""),
-				true,
-				false,
+				CDirentry::flag_dir | CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
 				_T(""),
-				CDirentry::timestamp_seconds,
-				wxDateTime(28, wxDateTime::Feb, 1998, 5, 23, 30),
-				false
+				wxDateTime(28, wxDateTime::Feb, 1998, 5, 23, 30)
 			},
 			DEFAULT
 		});
@@ -980,12 +834,9 @@ void CDirectoryListingParserTest::InitEntries()
 				1294495,
 				_T("-C--E-----FTP"),
 				_T("B BCC3I1 7670"),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
 				_T(""),
-				CDirentry::timestamp_time,
-				wxDateTime(13, wxDateTime::Jan, calcYear(1, 13), 7, 42),
-				false
+				wxDateTime(13, wxDateTime::Jan, calcYear(1, 13), 7, 42)
 			},
 			DEFAULT
 		});
@@ -999,12 +850,9 @@ void CDirectoryListingParserTest::InitEntries()
 				85920,
 				_T("d-ewrewr"),
 				_T("20.20"),
-				true,
-				false,
+				CDirentry::flag_dir | CDirentry::flag_timestamp_date,
 				_T(""),
-				CDirentry::timestamp_date,
-				wxDateTime(29, wxDateTime::Mar, 2007),
-				false
+				wxDateTime(29, wxDateTime::Mar, 2007)
 			},
 			DEFAULT
 		});
@@ -1017,12 +865,9 @@ void CDirectoryListingParserTest::InitEntries()
 				512,
 				_T("drwxr-xr-x"),
 				_T("user group"),
-				true,
-				false,
+				CDirentry::flag_dir | CDirentry::flag_timestamp_date,
 				_T(""),
-				CDirentry::timestamp_date,
-				wxDateTime(1, wxDateTime::Oct, 2004),
-				false
+				wxDateTime(1, wxDateTime::Oct, 2004)
 			},
 			DEFAULT
 		});
@@ -1034,12 +879,9 @@ void CDirectoryListingParserTest::InitEntries()
 				12,
 				_T("-r--r--r--"),
 				_T(""),
-				false,
-				false,
+				CDirentry::flag_timestamp_date,
 				_T(""),
-				CDirentry::timestamp_date,
-				wxDateTime(11, wxDateTime::Nov, 2005),
-				false
+				wxDateTime(11, wxDateTime::Nov, 2005)
 			},
 			DEFAULT
 		});
@@ -1052,12 +894,9 @@ void CDirectoryListingParserTest::InitEntries()
 				512,
 				_T("drwxr-xr-x"),
 				_T("root sys"),
-				true,
-				false,
+				CDirentry::flag_dir | CDirentry::flag_timestamp_date,
 				_T(""),
-				CDirentry::timestamp_date,
-				wxDateTime(6, wxDateTime::Jan, 2005),
-				false
+				wxDateTime(6, wxDateTime::Jan, 2005)
 			},
 			DEFAULT
 		});
@@ -1070,12 +909,9 @@ void CDirectoryListingParserTest::InitEntries()
 				36864,
 				_T(""),
 				_T("QPGMR"),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
 				_T(""),
-				CDirentry::timestamp_seconds,
-				wxDateTime(18, wxDateTime::Sep, 2006, 14, 21, 26),
-				false
+				wxDateTime(18, wxDateTime::Sep, 2006, 14, 21, 26)
 			},
 			DEFAULT
 		});
@@ -1088,12 +924,9 @@ void CDirectoryListingParserTest::InitEntries()
 				2048,
 				_T(""),
 				_T(""),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
 				_T(""),
-				CDirentry::timestamp_seconds,
-				wxDateTime(23, wxDateTime::Sep, 2005, 14, 57, 7),
-				false
+				wxDateTime(23, wxDateTime::Sep, 2005, 14, 57, 7)
 			},
 			DEFAULT
 		});
@@ -1106,12 +939,9 @@ void CDirectoryListingParserTest::InitEntries()
 				528,
 				_T("\"oooo\""),
 				_T("255, 0"),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
 				_T(""),
-				CDirentry::timestamp_seconds,
-				wxDateTime(6, wxDateTime::Apr, 2007, 14, 21, 18),
-				false
+				wxDateTime(6, wxDateTime::Apr, 2007, 14, 21, 18)
 			},
 			HPNONSTOP
 		});
@@ -1124,12 +954,9 @@ void CDirectoryListingParserTest::InitEntries()
 				528,
 				_T("\"oooo\""),
 				_T("255,255"),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
 				_T(""),
-				CDirentry::timestamp_seconds,
-				wxDateTime(6, wxDateTime::Apr, 2007, 14, 21, 18),
-				false
+				wxDateTime(6, wxDateTime::Apr, 2007, 14, 21, 18)
 			},
 			HPNONSTOP
 		});
@@ -1142,12 +969,9 @@ void CDirectoryListingParserTest::InitEntries()
 				1024,
 				_T("drwxr-xr-x"),
 				_T("user sys"),
-				true,
-				false,
+				CDirentry::flag_dir | CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
 				_T(""),
-				CDirentry::timestamp_time,
-				wxDateTime(30, wxDateTime::Jan, calcYear(1, 30), 12, 40),
-				false
+				wxDateTime(30, wxDateTime::Jan, calcYear(1, 30), 12, 40)
 			},
 			DEFAULT
 		});
@@ -1168,12 +992,9 @@ void CDirectoryListingParserTest::InitEntries()
 				100,
 				_T(""),
 				_T(""),
-				false,
-				false,
+				CDirentry::flag_timestamp_date,
 				_T(""),
-				CDirentry::timestamp_date,
-				wxDateTime(21, wxDateTime::May, 2003),
-				false
+				wxDateTime(21, wxDateTime::May, 2003)
 		},
 		DEFAULT
 	});
@@ -1185,12 +1006,9 @@ void CDirectoryListingParserTest::InitEntries()
 				-1,
 				_T(""),
 				_T(""),
-				true,
-				false,
+				CDirentry::flag_dir | CDirentry::flag_timestamp_date,
 				_T(""),
-				CDirentry::timestamp_date,
-				wxDateTime(04, wxDateTime::Mar, 2004),
-				false
+				wxDateTime(04, wxDateTime::Mar, 2004)
 		},
 		DEFAULT
 	});
@@ -1202,12 +1020,9 @@ void CDirectoryListingParserTest::InitEntries()
 				-1,
 				_T(""),
 				_T(""),
-				true,
-				false,
+				CDirentry::flag_dir,
 				_T(""),
-				CDirentry::timestamp_none,
-				wxDateTime(),
-				false
+				wxDateTime()
 		},
 		DEFAULT
 	});
@@ -1219,12 +1034,9 @@ void CDirectoryListingParserTest::InitEntries()
 				-1,
 				_T(""),
 				_T(""),
-				true,
-				false,
+				CDirentry::flag_dir | CDirentry::flag_timestamp_date,
 				_T(""),
-				CDirentry::timestamp_date,
-				wxDateTime(6, wxDateTime::Jun, 2005),
-				false
+				wxDateTime(6, wxDateTime::Jun, 2005)
 		},
 		DEFAULT
 	});
@@ -1236,12 +1048,9 @@ void CDirectoryListingParserTest::InitEntries()
 				-1,
 				_T(""),
 				_T(""),
-				false,
-				false,
+				0,
 				_T(""),
-				CDirentry::timestamp_none,
-				wxDateTime(),
-				false
+				wxDateTime()
 		},
 		DEFAULT
 	});
@@ -1260,12 +1069,9 @@ void CDirectoryListingParserTest::InitEntries()
 				-1,
 				_T(""),
 				_T(""),
-				false,
-				false,
+				0,
 				_T(""),
-				CDirentry::timestamp_none,
-				wxDateTime(),
-				false
+				wxDateTime()
 		},
 		MVS
 	});
@@ -1278,12 +1084,9 @@ void CDirectoryListingParserTest::InitEntries()
 				128,
 				_T(""),
 				_T(""),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time,
 				_T(""),
-				CDirentry::timestamp_time,
-				wxDateTime(22, wxDateTime::Jun, 2004, 16, 32),
-				false
+				wxDateTime(22, wxDateTime::Jun, 2004, 16, 32)
 		},
 		MVS
 	});
@@ -1296,12 +1099,9 @@ void CDirectoryListingParserTest::InitEntries()
 				45832,
 				_T(""),
 				_T(""),
-				false,
-				false,
+				0,
 				_T(""),
-				CDirentry::timestamp_none,
-				wxDateTime(),
-				false
+				wxDateTime()
 		},
 		MVS
 	});
@@ -1313,12 +1113,9 @@ void CDirectoryListingParserTest::InitEntries()
 				45832,
 				_T(""),
 				_T(""),
-				false,
-				false,
+				0,
 				_T(""),
-				CDirentry::timestamp_none,
-				wxDateTime(),
-				false
+				wxDateTime()
 			},
 			MVS
 		});
@@ -1331,12 +1128,9 @@ void CDirectoryListingParserTest::InitEntries()
 				-1,
 				_T(""),
 				_T(""),
-				false,
-				false,
+				0,
 				_T(""),
-				CDirentry::timestamp_none,
-				wxDateTime(),
-				false
+				wxDateTime()
 			},
 			MVS
 		});
@@ -1366,12 +1160,9 @@ void CDirectoryListingParserTest::InitEntries()
 				6955,
 				_T(""),
 				_T("060191"),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
 				_T(""),
-				CDirentry::timestamp_seconds,
-				wxDateTime(4, wxDateTime::Oct, 2005, 15, 28, 42),
-				false
+				wxDateTime(4, wxDateTime::Oct, 2005, 15, 28, 42)
 			},
 			ZVM
 		});
@@ -1383,12 +1174,9 @@ void CDirectoryListingParserTest::InitEntries()
 				512,
 				_T("drwxr-xr-x"),
 				_T("slopri devlab"),
-				true,
-				false,
+				CDirentry::flag_dir,
 				_T(""),
-				CDirentry::timestamp_none,
-				wxDateTime(),
-				false
+				wxDateTime()
 			},
 			DEFAULT
 		});
@@ -1402,12 +1190,9 @@ void CDirectoryListingParserTest::InitEntries()
 				1234,
 				_T(""),
 				_T(""),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
 				_T(""),
-				CDirentry::timestamp_seconds,
-				utc,
-				false
+				utc
 			},
 			DEFAULT
 		});
@@ -1423,12 +1208,9 @@ void CDirectoryListingParserTest::InitEntries()
 				-1,
 				_T(""),
 				_T(""),
-				false,
-				false,
+				0,
 				_T(""),
-				CDirentry::timestamp_none,
-				wxDateTime(),
-				false
+				wxDateTime()
 			},
 			MVS
 		});
@@ -1440,12 +1222,9 @@ void CDirectoryListingParserTest::InitEntries()
 				-1,
 				_T(""),
 				_T(""),
-				false,
-				false,
+				0,
 				_T(""),
-				CDirentry::timestamp_none,
-				wxDateTime(),
-				false
+				wxDateTime()
 			},
 			DEFAULT
 		});
@@ -1457,12 +1236,9 @@ void CDirectoryListingParserTest::InitEntries()
 				-1,
 				_T(""),
 				_T(""),
-				false,
-				false,
+				0,
 				_T(""),
-				CDirentry::timestamp_none,
-				wxDateTime(),
-				false
+				wxDateTime()
 			},
 			DEFAULT
 		});
@@ -1476,12 +1252,9 @@ void CDirectoryListingParserTest::InitEntries()
 				65718921,
 				_T("00"),
 				_T("1179 1179"),
-				false,
-				false,
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
 				_T(""),
-				CDirentry::timestamp_seconds,
-				utc,
-				false
+				utc
 			},
 			DEFAULT
 		});	
@@ -1493,12 +1266,9 @@ void CDirectoryListingParserTest::InitEntries()
 				-1,
 				_T(""),
 				_T(""),
-				true,
-				true,
+				CDirentry::flag_dir | CDirentry::flag_link,
 				_T("/foo"),
-				CDirentry::timestamp_none,
-				wxDateTime(),
-				false
+				wxDateTime()
 			},
 			DEFAULT
 		});	
@@ -1511,12 +1281,9 @@ void CDirectoryListingParserTest::InitEntries()
 				-1,
 				_T(""),
 				_T(""),
-				false,
-				false,
+				0,
 				_T(""),
-				CDirentry::timestamp_none,
-				wxDateTime(),
-				false
+				wxDateTime()
 			},
 			DEFAULT
 		});
@@ -1528,30 +1295,38 @@ void CDirectoryListingParserTest::InitEntries()
 				-1,
 				_T(""),
 				_T(""),
-				true,
-				true,
+				CDirentry::flag_dir | CDirentry::flag_link,
 				_T(""),
-				CDirentry::timestamp_none,
-				wxDateTime(),
-				false
+				wxDateTime()
 			},
 			DEFAULT
-		});	
+		});
+
+		utc = wxDateTime(22, wxDateTime::Jul, 2009, 9, 25, 10);
+		utc.MakeFromTimezone(wxDateTime::UTC);
+		m_entries.push_back((t_entry){
+			"size=1365694195;type=file;modify=20090722092510;\tadsl TV 2009-07-22 08-25-10 78 mlsd file that can get parsed as unix.file",
+			{
+				_T("adsl TV 2009-07-22 08-25-10 78 mlsd file that can get parsed as unix.file"),
+				1365694195,
+				_T(""),
+				_T(""),
+				CDirentry::flag_timestamp_date | CDirentry::flag_timestamp_time | CDirentry::flag_timestamp_seconds,
+				_T(""),
+				utc
+			},
+			DEFAULT
+		});
 
 /*
 	wxString name;
 	wxLongLong size;
 	wxString permissions;
 	wxString ownerGroup;
-	bool dir;
-	bool link;
+	int flags;
 	wxString target; // Set to linktarget it link is true
 
-	bool hasDate;
-	bool hasTime;
 	wxDateTime time;
-
-	bool unsure;
 */
 
 	// Fix line endings
